@@ -21,6 +21,14 @@
 #include "vtkObjectFactory.h"
 #include "vtkWindows.h"
 
+// OSAtomic.h optimizations only used in 10.5 and later
+#if defined(__APPLE__)
+  #include <AvailabilityMacros.h>
+  #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1050
+    #include <libkern/OSAtomic.h>
+  #endif
+#endif
+
 //-------------------------------------------------------------------------
 vtkTimeStamp* vtkTimeStamp::New()
 {
@@ -31,10 +39,26 @@ vtkTimeStamp* vtkTimeStamp::New()
 //-------------------------------------------------------------------------
 void vtkTimeStamp::Modified()
 {
+// Windows optimization
 #if defined(WIN32) || defined(_WIN32)
   static LONG vtkTimeStampTime = 0;
-
   this->ModifiedTime = (unsigned long)InterlockedIncrement(&vtkTimeStampTime);
+
+// Mac optimization
+#elif defined(__APPLE__) && (MAC_OS_X_VERSION_MIN_REQUIRED >= 1050)
+ #if __LP64__
+  // "ModifiedTime" is "unsigned long", a type that changess sizes
+  // depending on architecture.  The atomic increment is safe, since it
+  // operates on a variable of the exact type needed.  The cast does not
+  // change the size, but does change signedness, which is not ideal.
+  static volatile int64_t vtkTimeStampTime = 0;
+  this->ModifiedTime = (unsigned long)OSAtomicIncrement64Barrier(&vtkTimeStampTime);
+ #else
+  static volatile int32_t vtkTimeStampTime = 0;
+  this->ModifiedTime = (unsigned long)OSAtomicIncrement32Barrier(&vtkTimeStampTime);
+ #endif
+
+// General case
 #else
   static unsigned long vtkTimeStampTime = 0;
   static vtkSimpleCriticalSection TimeStampCritSec;
@@ -44,12 +68,3 @@ void vtkTimeStamp::Modified()
   TimeStampCritSec.Unlock();
 #endif
 }
-
-
-
-
-
-
-
-
-
