@@ -26,10 +26,12 @@
 #include "vtkQuad.h"
 #include "vtkTriangle.h"
 #include "vtkBox.h"
+#include "vtkMergePoints.h"
 
-vtkCxxRevisionMacro(vtkPolygon, "$Revision: 1.2 $");
+vtkCxxRevisionMacro(vtkPolygon, "$Revision: 1.8.2.1 $");
 vtkStandardNewMacro(vtkPolygon);
 
+//----------------------------------------------------------------------------
 // Instantiate polygon.
 vtkPolygon::vtkPolygon()
 {
@@ -45,6 +47,7 @@ vtkPolygon::vtkPolygon()
   this->Normal[0] = this->Normal[1] = this->Normal[2] = 0.0;
 }
 
+//----------------------------------------------------------------------------
 vtkPolygon::~vtkPolygon()
 {
   this->Tris->Delete();
@@ -60,6 +63,7 @@ vtkPolygon::~vtkPolygon()
 #define VTK_POLYGON_INTERSECTION 2
 #define VTK_POLYGON_ON_LINE 3
 
+//----------------------------------------------------------------------------
 //
 // In many of the functions that follow, the Points and PointIds members 
 // of the Cell are assumed initialized.  This is usually done indirectly
@@ -117,6 +121,58 @@ void vtkPolygon::ComputeNormal(vtkPoints *p, int numPts, vtkIdType *pts,
   vtkMath::Normalize(n);
 }
 
+//----------------------------------------------------------------------------
+// Compute the polygon normal from a points list, and a list of point ids
+// that index into the points list. This version will handle non-convex
+// polygons.
+void vtkPolygon::ComputeNormal(vtkIdTypeArray *ids, vtkPoints *p, double n[3])
+{
+  vtkIdType i, numPts = ids->GetNumberOfTuples();
+  double v0[3], v1[3], v2[3];
+  double ax, ay, az, bx, by, bz;
+// 
+// Check for special triangle case. Saves extra work.
+// 
+  n[0] = n[1] = n[2] = 0.0;
+  if ( numPts == 2 || numPts == 1 ) 
+    {
+    return;
+    }
+
+  if ( numPts == 3 ) 
+    {
+    p->GetPoint(ids->GetValue(0),v0);
+    p->GetPoint(ids->GetValue(1),v1);
+    p->GetPoint(ids->GetValue(2),v2);
+    vtkTriangle::ComputeNormal(v0, v1, v2, n);
+    return;
+    }
+
+  //  Because polygon may be concave, need to accumulate cross products to 
+  //  determine true normal.
+  //
+  p->GetPoint(ids->GetValue(0),v1); //set things up for loop
+  p->GetPoint(ids->GetValue(1),v2);
+
+  for (i=0; i < numPts; i++) 
+    {
+    v0[0] = v1[0]; v0[1] = v1[1]; v0[2] = v1[2];
+    v1[0] = v2[0]; v1[1] = v2[1]; v1[2] = v2[2];
+    p->GetPoint(ids->GetValue((i+2)%numPts),v2);
+
+    // order is important!!! to maintain consistency with polygon vertex order 
+    ax = v2[0] - v1[0]; ay = v2[1] - v1[1]; az = v2[2] - v1[2];
+    bx = v0[0] - v1[0]; by = v0[1] - v1[1]; bz = v0[2] - v1[2];
+
+    n[0] += (ay * bz - az * by);
+    n[1] += (az * bx - ax * bz);
+    n[2] += (ax * by - ay * bx);
+    }
+
+  vtkMath::Normalize(n);
+}
+
+//----------------------------------------------------------------------------
 // Compute the polygon normal from a list of doubleing points. This version
 // will handle non-convex polygons.
 void vtkPolygon::ComputeNormal(vtkPoints *p, double *n)
@@ -151,6 +207,7 @@ void vtkPolygon::ComputeNormal(vtkPoints *p, double *n)
   vtkMath::Normalize(n);
 }
 
+//----------------------------------------------------------------------------
 // Compute the polygon normal from an array of points. This version assumes
 // that the polygon is convex, and looks for the first valid normal.
 void vtkPolygon::ComputeNormal (int numPts, double *pts, double n[3])
@@ -168,7 +225,7 @@ void vtkPolygon::ComputeNormal (int numPts, double *pts, double n[3])
   v2 = pts + 3;
   v3 = pts + 6;
 
-  for (i=0; i<numPts-2; i++) 
+  for (i=0; i<numPts-2; i++)
     {
     ax = v2[0] - v1[0]; ay = v2[1] - v1[1]; az = v2[2] - v1[2];
     bx = v3[0] - v1[0]; by = v3[1] - v1[1]; bz = v3[2] - v1[2];
@@ -178,13 +235,13 @@ void vtkPolygon::ComputeNormal (int numPts, double *pts, double n[3])
     n[2] = (ax * by - ay * bx);
 
     length = sqrt (n[0] * n[0] + n[1] * n[1] + n[2] * n[2]);
-    if (length != 0.0) 
+    if (length != 0.0)
       {
       n[0] /= length;
       n[1] /= length;
       n[2] /= length;
       return;
-      } 
+      }
     else
       {
       v1 = v2;
@@ -194,8 +251,9 @@ void vtkPolygon::ComputeNormal (int numPts, double *pts, double n[3])
     } //over all points
 }
 
+//----------------------------------------------------------------------------
 int vtkPolygon::EvaluatePosition(double x[3], double* closestPoint,
-                                 int& vtkNotUsed(subId), double pcoords[3], 
+                                 int& vtkNotUsed(subId), double pcoords[3],
                                  double& minDist2, double *weights)
 {
   int i;
@@ -203,7 +261,7 @@ int vtkPolygon::EvaluatePosition(double x[3], double* closestPoint,
   double ray[3];
 
   this->ParameterizePolygon(p0, p10, l10, p20, l20, n);
-  this->ComputeWeights(x,weights);
+  this->InterpolateFunctions(x,weights);
   vtkPlane::ProjectPoint(x,p0,n,cp);
 
   for (i=0; i<3; i++)
@@ -215,10 +273,10 @@ int vtkPolygon::EvaluatePosition(double x[3], double* closestPoint,
 
   if ( pcoords[0] >= 0.0 && pcoords[0] <= 1.0 &&
        pcoords[1] >= 0.0 && pcoords[1] <= 1.0 &&
-       (this->PointInPolygon(cp, this->Points->GetNumberOfPoints(), 
-                             ((vtkDoubleArray *)this->Points->GetData())
-                             ->GetPointer(0), this->GetBounds(),n)
-        == VTK_POLYGON_INSIDE) )
+       (this->PointInPolygon(cp, this->Points->GetNumberOfPoints(),
+                             static_cast<vtkDoubleArray *>(
+                               this->Points->GetData())->GetPointer(0),
+                             this->GetBounds(),n) == VTK_POLYGON_INSIDE) )
     {
     if (closestPoint)
       {
@@ -249,8 +307,8 @@ int vtkPolygon::EvaluatePosition(double x[3], double* closestPoint,
         dist2 = vtkLine::DistanceToLine(x, pt1, pt2, t, closest);
         if ( dist2 < minDist2 )
           {
-          closestPoint[0] = closest[0]; 
-          closestPoint[1] = closest[1]; 
+          closestPoint[0] = closest[0];
+          closestPoint[1] = closest[1];
           closestPoint[2] = closest[2];
           minDist2 = dist2;
           }
@@ -260,7 +318,8 @@ int vtkPolygon::EvaluatePosition(double x[3], double* closestPoint,
     }
 }
 
-void vtkPolygon::EvaluateLocation(int& vtkNotUsed(subId), double pcoords[3], 
+//----------------------------------------------------------------------------
+void vtkPolygon::EvaluateLocation(int& vtkNotUsed(subId), double pcoords[3],
                                   double x[3], double *weights)
 {
   int i;
@@ -272,9 +331,51 @@ void vtkPolygon::EvaluateLocation(int& vtkNotUsed(subId), double pcoords[3],
     x[i] = p0[i] + pcoords[0]*p10[i] + pcoords[1]*p20[i];
     }
 
-  this->ComputeWeights(x,weights);
+  this->InterpolateFunctions(x,weights);
 }
 
+//----------------------------------------------------------------------------
+// Compute interpolation weights using 1/r**2 normalized sum.
+void vtkPolygon::InterpolateFunctions(double x[3], double *weights)
+{
+  int i;
+  int numPts=this->Points->GetNumberOfPoints();
+  double sum, pt[3];
+
+  for (sum=0.0, i=0; i<numPts; i++)
+    {
+    this->Points->GetPoint(i, pt);
+    weights[i] = vtkMath::Distance2BetweenPoints(x,pt);
+    if ( weights[i] == 0.0 ) //exact hit
+      {
+      for (int j=0; j<numPts; j++)
+        {
+        weights[j] = 0.0;
+        }
+      weights[i] = 1.0;
+      return;
+      }
+    else
+      {
+      weights[i] = 1.0 / (weights[i]*weights[i]);
+      sum += weights[i];
+      }
+    }
+
+  for (i=0; i<numPts; i++)
+    {
+    weights[i] /= sum;
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkPolygon::InterpolateDerivs(double pcoords[3], double *derivs)
+{
+  (void)pcoords;
+  (void)derivs;
+}
+
+//----------------------------------------------------------------------------
 // Create a local s-t coordinate system for a polygon. The point p0 is
 // the origin of the local system, p10 is s-axis vector, and p20 is the 
 // t-axis vector. (These are expressed in the modelling coordinate system and
@@ -364,6 +465,7 @@ int vtkPolygon::ParameterizePolygon(double *p0, double *p10, double& l10,
 #define TRUE 1
 #endif
 
+//----------------------------------------------------------------------------
 // Determine whether point is inside polygon. Function uses ray-casting
 // to determine if point is inside polygon. Works for arbitrary polygon shape
 // (e.g., non-convex). Returns 0 if point is not in polygon; 1 if it is.
@@ -539,6 +641,7 @@ int vtkPolygon::PointInPolygon (double x[3], int numPts, double *pts,
 
 #define VTK_POLYGON_TOLERANCE 1.0e-06
 
+//----------------------------------------------------------------------------
 // Triangulate polygon. 
 //
 int vtkPolygon::Triangulate(vtkIdList *outTris)
@@ -559,13 +662,169 @@ int vtkPolygon::Triangulate(vtkIdList *outTris)
   
   if ( !success ) //degenerate triangle encountered
     {
-    vtkWarningMacro(<< "Degenerate polygon encountered during triangulation");
+    vtkDebugMacro(<< "Degenerate polygon encountered during triangulation");
     }
 
   outTris->DeepCopy(this->Tris);
   return success;
 }
 
+//----------------------------------------------------------------------------
+// Split into non-degenerate polygons prior to triangulation
+//
+int vtkPolygon::NonDegenerateTriangulate(vtkIdList *outTris)
+{
+  double pt[3], bounds[6];
+  vtkIdType ptId, numPts;
+
+  // ComputeBounds does not give the correct bounds
+  // So we do it manually
+  bounds[0]= VTK_DOUBLE_MAX;
+  bounds[1]=-VTK_DOUBLE_MAX;
+  bounds[2]= VTK_DOUBLE_MAX;
+  bounds[3]=-VTK_DOUBLE_MAX;
+  bounds[4]= VTK_DOUBLE_MAX;
+  bounds[5]=-VTK_DOUBLE_MAX;
+
+  numPts = this->GetNumberOfPoints();
+
+  for(int i=0; i<numPts; i++)
+    {
+    this->Points->GetPoint(i,pt);
+
+    if(pt[0] < bounds[0]) { bounds[0] = pt[0]; }
+    if(pt[1] < bounds[2]) { bounds[2] = pt[1]; }
+    if(pt[2] < bounds[4]) { bounds[4] = pt[2]; }
+    if(pt[0] > bounds[1]) { bounds[1] = pt[0]; }
+    if(pt[1] > bounds[3]) { bounds[3] = pt[1]; }
+    if(pt[2] > bounds[5]) { bounds[5] = pt[2]; }
+    }
+
+  outTris->Reset();
+  outTris->Allocate(3*(2*numPts-4));
+
+  vtkPoints *newPts = vtkPoints::New();
+  newPts->Allocate(numPts);
+
+  vtkMergePoints *mergePoints = vtkMergePoints::New();
+  mergePoints->InitPointInsertion(newPts,bounds);
+  mergePoints->SetDivisions(10,10,10);
+
+  vtkIdTypeArray *matchingIds = vtkIdTypeArray::New();
+  matchingIds->SetNumberOfTuples(numPts);
+
+  int numDuplicatePts = 0;
+
+  for(int i=0; i<numPts; i++)
+    {
+    this->Points->GetPoint(i,pt);
+    if(mergePoints->InsertUniquePoint(pt,ptId))
+      {
+      matchingIds->SetValue(i,ptId+numDuplicatePts);
+      }
+    else
+      {
+      matchingIds->SetValue(i,ptId+numDuplicatePts);
+      numDuplicatePts++;
+      }
+    }
+
+  mergePoints->Delete();
+  newPts->Delete();
+ 
+  int numPtsRemoved = 0;
+  vtkIdType tri[3];
+
+  while(numPtsRemoved < numPts)
+    {
+    vtkIdType start = 0;
+    vtkIdType end = numPts-1;
+
+    for(;start<numPts;start++)
+      {
+      if(matchingIds->GetValue(start) >= 0)
+        {
+        break;
+        }
+      }
+
+    if(start >= end)
+      {
+      vtkErrorMacro("ERROR: start >= end");
+      break;
+      }
+
+    for(int i=start; i<numPts; i++)
+      {
+      if(matchingIds->GetValue(i) < 0)
+        {
+        continue;
+        }
+
+      if(matchingIds->GetValue(i) != i)
+        {
+        start = (matchingIds->GetValue(i) + 1) % numPts;
+        end = i;
+ 
+        while(matchingIds->GetValue(start) < 0)
+          {
+          start++;
+          }
+
+        break;
+        }
+      }
+
+    vtkPolygon *polygon = vtkPolygon::New();
+    polygon->Points->SetDataTypeToDouble();
+
+    int numPolygonPts = start<end ? end-start+1 : end-start+numPts+1;
+
+    for(int i=0; i<numPolygonPts; i++)
+      {
+      ptId = start+i;
+
+      if(matchingIds->GetValue(ptId) >= 0)
+        {
+        numPtsRemoved++;
+        matchingIds->SetValue(ptId,-1);
+
+        polygon->PointIds->InsertNextId(ptId);
+        polygon->Points->InsertNextPoint(this->Points->GetPoint(ptId));
+        }
+      }
+
+    vtkIdList *outTriangles = vtkIdList::New();
+    outTriangles->Allocate(3*(2*polygon->GetNumberOfPoints()-4));
+
+    polygon->Triangulate(outTriangles);
+
+    int outNumTris = outTriangles->GetNumberOfIds();
+
+    for(int i=0; i<outNumTris; i+=3)
+      {
+      tri[0] = outTriangles->GetId(i);
+      tri[1] = outTriangles->GetId(i+1);
+      tri[2] = outTriangles->GetId(i+2);
+
+      tri[0] = polygon->PointIds->GetId(tri[0]);
+      tri[1] = polygon->PointIds->GetId(tri[1]);
+      tri[2] = polygon->PointIds->GetId(tri[2]);
+
+      outTris->InsertNextId(tri[0]);
+      outTris->InsertNextId(tri[1]);
+      outTris->InsertNextId(tri[2]);
+      }
+
+    polygon->Delete();
+    outTriangles->Delete();
+    }
+
+  matchingIds->Delete();
+  return 1;
+}
+
+//----------------------------------------------------------------------------
 // Special structures for building loops. This is a double-linked list.
 typedef struct _vtkPolyVertex 
   {
@@ -592,6 +851,7 @@ public:
   double Normal[3];
 };
     
+//----------------------------------------------------------------------------
 // tolerance is squared
 vtkPolyVertexList::vtkPolyVertexList(vtkIdList *ptIds, vtkPoints *pts,
                                      double tol2)
@@ -646,11 +906,13 @@ vtkPolyVertexList::vtkPolyVertexList(vtkIdList *ptIds, vtkPoints *pts,
     }
 }
 
+//----------------------------------------------------------------------------
 vtkPolyVertexList::~vtkPolyVertexList()
 {
   delete [] this->Array;
 }
 
+//----------------------------------------------------------------------------
 // Remove the vertex from the polygon (forming a triangle with
 // its previous and next neighbors, and reinsert the neighbors
 // into the priority queue.
@@ -691,6 +953,7 @@ void vtkPolyVertexList::RemoveVertex(int i, vtkIdList *tris,
     }
 }
 
+//----------------------------------------------------------------------------
 int vtkPolyVertexList::ComputeNormal()
 {
   vtkLocalPolyVertex *vtx=this->Head;
@@ -720,6 +983,7 @@ int vtkPolyVertexList::ComputeNormal()
     }
 }
   
+//----------------------------------------------------------------------------
 // The measure is the ratio of triangle perimeter^2 to area;
 // the sign of the measure is determined by dotting the local
 // vector with the normal (concave features return a negative
@@ -751,6 +1015,7 @@ double vtkPolyVertexList::ComputeMeasure(vtkLocalPolyVertex *vtx)
     }
 }
 
+//----------------------------------------------------------------------------
 // returns != 0 if vertex can be removed. Uses half-space
 // comparison to determine whether ear-cut is valid, and may
 // resort to line-plane intersections to resolve possible
@@ -825,6 +1090,7 @@ int vtkPolyVertexList::CanRemoveVertex(int id, double tolerance)
     }
 }
 
+//----------------------------------------------------------------------------
 // Triangulation method based on ear-cutting. Triangles, or ears, are
 // cut off from the polygon based on the angle of the vertex. Small
 // angles (narrow triangles) are cut off first. This implementation uses
@@ -898,6 +1164,7 @@ int vtkPolygon::EarCutTriangulation ()
 }
 
 
+//----------------------------------------------------------------------------
 int vtkPolygon::CellBoundary(int vtkNotUsed(subId), double pcoords[3], 
                              vtkIdList *pts)
 {
@@ -919,7 +1186,7 @@ int vtkPolygon::CellBoundary(int vtkNotUsed(subId), double pcoords[3],
 
   //find edge with largest and next largest weight values. This will be
   //the closest edge.
-  this->ComputeWeights(x,weights);
+  this->InterpolateFunctions(x,weights);
   for ( i=0; i < numPts; i++ )
     {
     if ( weights[i] > largestWeight )
@@ -956,9 +1223,9 @@ int vtkPolygon::CellBoundary(int vtkNotUsed(subId), double pcoords[3],
   if ( pcoords[0] >= 0.0 && pcoords[0] <= 1.0 &&
        pcoords[1] >= 0.0 && pcoords[1] <= 1.0 &&
        (this->PointInPolygon(x, this->Points->GetNumberOfPoints(), 
-                             ((vtkDoubleArray *)this->Points->GetData())
-                             ->GetPointer(0), this->GetBounds(),n)
-        == VTK_POLYGON_INSIDE) )
+                             static_cast<vtkDoubleArray *>(
+                               this->Points->GetData())->GetPointer(0),
+                             this->GetBounds(),n) == VTK_POLYGON_INSIDE) )
     {
     return 1;
     }
@@ -968,6 +1235,7 @@ int vtkPolygon::CellBoundary(int vtkNotUsed(subId), double pcoords[3],
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkPolygon::Contour(double value, vtkDataArray *cellScalars, 
                          vtkPointLocator *locator,
                          vtkCellArray *verts, vtkCellArray *lines, 
@@ -1027,6 +1295,7 @@ void vtkPolygon::Contour(double value, vtkDataArray *cellScalars,
     }
 }
 
+//----------------------------------------------------------------------------
 vtkCell *vtkPolygon::GetEdge(int edgeId)
 {
   int numPts=this->Points->GetNumberOfPoints();
@@ -1042,41 +1311,7 @@ vtkCell *vtkPolygon::GetEdge(int edgeId)
   return this->Line;
 }
 
-//
-// Compute interpolation weights using 1/r**2 normalized sum.
-//
-void vtkPolygon::ComputeWeights(double x[3], double *weights)
-{
-  int i;
-  int numPts=this->Points->GetNumberOfPoints();
-  double sum, pt[3];
-
-  for (sum=0.0, i=0; i<numPts; i++)
-    {
-    this->Points->GetPoint(i, pt);
-    weights[i] = vtkMath::Distance2BetweenPoints(x,pt);
-    if ( weights[i] == 0.0 ) //exact hit
-      {
-      for (int j=0; j<numPts; j++)
-        {
-        weights[j] = 0.0;
-        }
-      weights[i] = 1.0;
-      return;
-      }
-    else
-      {
-      weights[i] = 1.0 / (weights[i]*weights[i]);
-      sum += weights[i];
-      }
-    }
-
-  for (i=0; i<numPts; i++)
-    {
-    weights[i] /= sum;
-    }
-}
-
+//----------------------------------------------------------------------------
 //
 // Intersect this plane with finite line defined by p1 & p2 with tolerance tol.
 //
@@ -1121,6 +1356,7 @@ int vtkPolygon::IntersectWithLine(double p1[3], double p2[3], double tol,double&
 
 }
 
+//----------------------------------------------------------------------------
 int vtkPolygon::Triangulate(int vtkNotUsed(index), vtkIdList *ptIds, 
                             vtkPoints *pts)
 {
@@ -1155,13 +1391,14 @@ int vtkPolygon::Triangulate(int vtkNotUsed(index), vtkIdList *ptIds,
   return this->SuccessfulTriangulation;
 }
 
-// Samples at three points to compute derivatives in local r-s coordinate 
+//----------------------------------------------------------------------------
+// Samples at three points to compute derivatives in local r-s coordinate
 // system and projects vectors into 3D model coordinate system.
 // Note that the results are usually inaccurate because
 // this method actually returns the derivative of the interpolation
 // function  which  is obtained using 1/r**2 normalized sum.
 #define VTK_SAMPLE_DISTANCE 0.01
-void vtkPolygon::Derivatives(int vtkNotUsed(subId), double pcoords[3], 
+void vtkPolygon::Derivatives(int vtkNotUsed(subId), double pcoords[3],
                              double *values, int dim, double *derivs)
 {
   int i, j, k, idx;
@@ -1208,16 +1445,16 @@ void vtkPolygon::Derivatives(int vtkNotUsed(subId), double pcoords[3],
   for (i=0; i<3; i++)
     {
     x[0][i] = p0[i] + pcoords[0]*p10[i] + pcoords[1]*p20[i];
-    x[1][i] = p0[i] + (pcoords[0]+VTK_SAMPLE_DISTANCE)*p10[i] + 
+    x[1][i] = p0[i] + (pcoords[0]+VTK_SAMPLE_DISTANCE)*p10[i] +
               pcoords[1]*p20[i];
-    x[2][i] = p0[i] + pcoords[0]*p10[i] + 
+    x[2][i] = p0[i] + pcoords[0]*p10[i] +
               (pcoords[1]+VTK_SAMPLE_DISTANCE)*p20[i];
     }
 
   //for each sample point, sample data values
   for ( idx=0, k=0; k < 3; k++ ) //loop over three sample points
     {
-    this->ComputeWeights(x[k],weights);
+    this->InterpolateFunctions(x[k],weights);
     for ( j=0; j < dim; j++, idx++) //over number of derivates requested
       {
       sample[idx] = 0.0;
@@ -1254,7 +1491,8 @@ void vtkPolygon::Derivatives(int vtkNotUsed(subId), double pcoords[3],
   delete [] sample;
 }
 
-void vtkPolygon::Clip(double value, vtkDataArray *cellScalars, 
+//----------------------------------------------------------------------------
+void vtkPolygon::Clip(double value, vtkDataArray *cellScalars,
                       vtkPointLocator *locator, vtkCellArray *tris,
                       vtkPointData *inPD, vtkPointData *outPD,
                       vtkCellData *inCD, vtkIdType cellId, vtkCellData *outCD,
@@ -1299,12 +1537,13 @@ void vtkPolygon::Clip(double value, vtkDataArray *cellScalars,
       this->TriScalars->SetTuple(1,cellScalars->GetTuple(p2));
       this->TriScalars->SetTuple(2,cellScalars->GetTuple(p3));
 
-      this->Triangle->Clip(value, this->TriScalars, locator, tris, 
+      this->Triangle->Clip(value, this->TriScalars, locator, tris,
                           inPD, outPD, inCD, cellId, outCD, insideOut);
       }
     }
 }
 
+//----------------------------------------------------------------------------
 // Method intersects two polygons. You must supply the number of points and
 // point coordinates (npts, *pts) and the bounding box (bounds) of the two
 // polygons. Also supply a tolerance squared for controlling
@@ -1312,7 +1551,7 @@ void vtkPolygon::Clip(double value, vtkDataArray *cellScalars,
 // not. A single point of intersection x[3] is also returned if there
 // is an intersection.
 int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[6],
-                                            int npts2, double *pts2, 
+                                            int npts2, double *pts2,
                                             double bounds2[6], double tol2,
                                             double x[3])
 {
@@ -1325,7 +1564,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
   //
   vtkPolygon::ComputeNormal(npts2, pts2, n);
 
-  for (i=0; i<npts; i++) 
+  for (i=0; i<npts; i++)
     {
     p1 = pts + 3*i;
     p2 = pts + 3*((i+1)%npts);
@@ -1339,7 +1578,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
       continue;
       }
 
-    if ( (vtkPlane::IntersectWithLine(p1,p2,n,pts2,t,x)) == 1 ) 
+    if ( (vtkPlane::IntersectWithLine(p1,p2,n,pts2,t,x)) == 1 )
       {
       if ( (npts2==3
             && vtkTriangle::PointInTriangle(x,pts2,pts2+3,pts2+6,tol2))
@@ -1349,7 +1588,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
         {
         return 1;
         }
-      } 
+      }
     else
       {
       return 0;
@@ -1360,7 +1599,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
   //
   vtkPolygon::ComputeNormal(npts, pts, n);
 
-  for (i=0; i<npts2; i++) 
+  for (i=0; i<npts2; i++)
     {
     p1 = pts2 + 3*i;
     p2 = pts2 + 3*((i+1)%npts2);
@@ -1375,7 +1614,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
       continue;
       }
 
-    if ( (vtkPlane::IntersectWithLine(p1,p2,n,pts,t,x)) == 1 ) 
+    if ( (vtkPlane::IntersectWithLine(p1,p2,n,pts,t,x)) == 1 )
       {
       if ( (npts==3 && vtkTriangle::PointInTriangle(x,pts,pts+3,pts+6,tol2))
            || (npts>3 && vtkPolygon::PointInPolygon(x,npts,pts,bounds,n)
@@ -1383,7 +1622,7 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
         {
         return 1;
         }
-      } 
+      }
     else
       {
       return 0;
@@ -1393,14 +1632,83 @@ int vtkPolygon::IntersectPolygonWithPolygon(int npts, double *pts,double bounds[
   return 0;
 }
 
+//----------------------------------------------------------------------------
+// Compute the area of the polygon (oriented in 3D space). It uses an
+// efficient approach where the area is computed in 2D and then projected into
+// 3D space.
+double vtkPolygon::ComputeArea(vtkPoints *p, vtkIdType numPts, vtkIdType *pts,
+                               double n[3])
+{
+  if ( numPts < 3 )
+    {
+    return 0.0;
+    }
+  else
+    {
+    double area=0.0;
+    double nx, ny, nz;
+    int coord, i;
+
+    vtkPolygon::ComputeNormal(p,numPts, pts, n);
+
+    // Select the projection direction
+    nx = (n[0] > 0.0 ? n[0] : -n[0]);     // abs x-coord
+    ny = (n[1] > 0.0 ? n[1] : -n[1]);     // abs y-coord
+    nz = (n[2] > 0.0 ? n[2] : -n[2]);     // abs z-coord
+
+    coord = (nx > ny ? (nx > nz ? 0 : 2) : (ny > nz ? 1 : 2));
+
+    // compute area of the 2D projection
+    double x0[3], x1[3], x2[3], *v0, *v1, *v2;
+    v0 = x0;
+    v1 = x1;
+    v2 = x2;
+
+    for (i=0; i<numPts; i++)
+      {
+      p->GetPoint(pts[i],v0);
+      p->GetPoint(pts[(i+1)%numPts],v1);
+      p->GetPoint(pts[(i+2)%numPts],v2);
+      switch (coord) 
+        {
+        case 0:
+          area += v1[1] * (v2[2] - v0[2]);
+          continue;
+        case 1:
+          area += v1[0] * (v2[2] - v0[2]);
+          continue;
+        case 2:
+          area += v1[0] * (v2[1] - v0[1]);
+          continue;
+        }
+      }
+
+    // scale to get area before projection
+    switch (coord) 
+      {
+      case 0:
+        area /= (2.0*nx);
+        break;
+      case 1:
+        area /= (2.0*ny);
+        break;
+      case 2:
+        area /= (2.0*nz);
+      }
+    return fabs(area);
+    }//general polygon
+}
+
+
 //----------------------------------------------------------------------------
 void vtkPolygon::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-  
+
   os << indent << "Tolerance: " << this->Tolerance << "\n";
-  os << indent << "SuccessfulTriangulation: " << this->SuccessfulTriangulation << "\n";
-  os << indent << "Normal: (" << this->Normal[0] << ", " 
+  os << indent << "SuccessfulTriangulation: " <<
+    this->SuccessfulTriangulation << "\n";
+  os << indent << "Normal: (" << this->Normal[0] << ", "
      << this->Normal[1] << ", " << this->Normal[2] << ")\n";
   os << indent << "Tris:\n";
   this->Tris->PrintSelf(os,indent.GetNextIndent());
@@ -1413,3 +1721,145 @@ void vtkPolygon::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Line:\n";
   this->Line->PrintSelf(os,indent.GetNextIndent());
 }
+
+//----------------------------------------------------------------------------
+// Compute the polygon centroid from a points list, and a list of point ids
+// that index into the points list. 
+void vtkPolygon::ComputeCentroid(vtkIdTypeArray *ids,
+                                 vtkPoints *p,
+                                 double c[3])
+{
+  vtkIdType i, numPts = ids->GetNumberOfTuples();
+  double p0[3];
+  double a = 1.0 / static_cast<double>(numPts);
+  c[0] = c[1] = c[2] = 0.0;
+  for (i=0; i < numPts; i++) 
+    {
+    p->GetPoint(ids->GetValue(i),p0);
+
+    c[0] += p0[0];
+    c[1] += p0[1];
+    c[2] += p0[2];
+    }
+
+  c[0] *= a;
+  c[1] *= a;
+  c[2] *= a;
+}
+
+//----------------------------------------------------------------------------
+double vtkPolygon::DistanceToPolygon(double x[3], int numPts, double *pts,
+                                     double bounds[6], double closest[3])
+{
+  // First check to see if the point is inside the polygon
+  // do a quick bounds check
+  if ( x[0] >= bounds[0] && x[0] <= bounds[1] &&
+       x[1] >= bounds[2] && x[1] <= bounds[3] &&
+       x[2] >= bounds[4] && x[2] <= bounds[5])
+    {
+    double n[3];
+    vtkPolygon::ComputeNormal(numPts, pts, n);
+    if ( vtkPolygon::PointInPolygon(x,numPts,pts,bounds,n) )
+      {
+      closest[0] = x[0];
+      closest[1] = x[1];
+      closest[2] = x[2];
+      return 0.0;
+      }
+    }
+  
+  // Not inside, compute the distance of the point to the edges.
+  double minDist2=VTK_LARGE_FLOAT;
+  double *p0, *p1, dist2, t, c[3];
+  for (int i=0; i<numPts; i++)
+    {
+    p0 = pts + 3*i;
+    p1 = pts + 3*((i+1)%numPts);
+    dist2 = vtkLine::DistanceToLine(x, p0, p1, t, c);
+    if ( dist2 < minDist2 )
+      {
+      minDist2 = dist2;
+      closest[0] = c[0];
+      closest[1] = c[1];
+      closest[2] = c[2];
+      }
+    }
+
+  return sqrt(minDist2);
+}
+
+//----------------------------------------------------------------------------
+int vtkPolygon::IntersectConvex2DCells(vtkCell *cell1, vtkCell *cell2,
+                                       double tol, double p0[3], double p1[3])
+{
+  // Intersect the six total edges of the two triangles against each other. Two points are
+  // all that are required.
+  double *x[2], pcoords[3], t, x0[3], x1[3];
+  x[0] = p0; x[1] = p1;
+  int subId, idx=0;
+  double t2 = tol*tol;
+
+  // Loop over edges of second polygon and intersect against first polygon
+  vtkIdType i, numPts = cell2->Points->GetNumberOfPoints();
+  for (i=0; i<numPts; i++)
+    {
+    cell2->Points->GetPoint(i,x0);
+    cell2->Points->GetPoint((i+1)%numPts,x1);
+
+    if ( cell1->IntersectWithLine(x0,x1,tol,t,x[idx],pcoords,subId) )
+      {
+      if ( idx == 0 )
+        {
+        idx++;
+        }
+      else if ( ((x[1][0]-x[0][0])*(x[1][0]-x[0][0]) + (x[1][1]-x[0][1])*(x[1][1]-x[0][1]) + 
+                 (x[1][2]-x[0][2])*(x[1][2]-x[0][2])) > t2 )
+        {
+        return 2;
+        }
+      }//if edge intersection
+    }//over all edges
+  
+  // Loop over edges of first polygon and intersect against second polygon
+  numPts = cell1->Points->GetNumberOfPoints();
+  for (i=0; i<numPts; i++)
+    {
+    cell1->Points->GetPoint(i,x0);
+    cell1->Points->GetPoint((i+1)%numPts,x1);
+
+    if ( cell2->IntersectWithLine(x0,x1,tol,t,x[idx],pcoords,subId) )
+      {
+      if ( idx == 0 )
+        {
+        idx++;
+        }
+      else if ( ((x[1][0]-x[0][0])*(x[1][0]-x[0][0]) + (x[1][1]-x[0][1])*(x[1][1]-x[0][1]) + 
+                 (x[1][2]-x[0][2])*(x[1][2]-x[0][2])) > t2 )
+        {
+        return 2;
+        }
+      }//if edge intersection
+    }//over all edges
+  
+  // Evaluate what we got
+  if ( idx == 1 )
+    {
+    return 1; //everything intersecting at single point
+    }
+  else
+    {
+    return 0;
+    }
+}
+
+//----------------------------------------------------------------------------
+#ifndef VTK_LEGACY_REMOVE
+void vtkPolygon::ComputeWeights(double x[3], double *weights)
+{
+  VTK_LEGACY_REPLACED_BODY(vtkPolygon::ComputeWeights, "VTK 5.2",
+                           vtkPolygon::InterpolateFunctions);
+  this->InterpolateFunctions(x, weights);
+}
+#endif
+
+
