@@ -16,19 +16,24 @@
 
 #include "vtkDataArrayCollection.h"
 #include "vtkFloatArray.h"
+#include "vtkMultiBlockDataSet.h"
 #include "vtkIdList.h"
 #include "vtkIdListCollection.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkSmartPointer.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStructuredGrid.h"
 #include "vtkStructuredPoints.h"
 #include "vtkUnstructuredGrid.h"
 
+#include <vtkstd/algorithm>
 #include <vtkstd/string>
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkEnSightReader, "$Revision: 1.58 $");
+vtkCxxRevisionMacro(vtkEnSightReader, "$Revision: 1.76 $");
 
 //----------------------------------------------------------------------------
 typedef vtkstd::vector< vtkSmartPointer<vtkIdList> > vtkEnSightReaderCellIdsTypeBase;
@@ -40,6 +45,7 @@ vtkEnSightReader::vtkEnSightReader()
   this->MeasuredFileName = NULL;
   this->MatchFileName = NULL;
 
+  this->ParticleCoordinatesByIndex = 0;
   this->IS = NULL;
   
   this->VariableMode = -1;
@@ -83,7 +89,6 @@ vtkEnSightReader::vtkEnSightReader()
 
   this->NumberOfMeasuredPoints = 0;
   
-  this->OutputsAreValid = 1;
   this->InitialRead = 1;
   this->NumberOfNewOutputs = 0;
 }
@@ -159,13 +164,50 @@ vtkEnSightReader::~vtkEnSightReader()
   this->TimeSets = NULL;
   this->FileSets->Delete();
   this->FileSets = NULL;
+
+  this->ActualTimeValue = 0.0;
 }
 
 //----------------------------------------------------------------------------
-void vtkEnSightReader::Execute()
+int vtkEnSightReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
   vtkDebugMacro("In execute ");
 
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+  vtkMultiBlockDataSet *output = vtkMultiBlockDataSet::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+
+  int tsLength =
+    outInfo->Length(vtkStreamingDemandDrivenPipeline::TIME_STEPS());
+  double* steps =
+    outInfo->Get(vtkStreamingDemandDrivenPipeline::TIME_STEPS());
+    
+  this->ActualTimeValue = this->TimeValue;
+
+  // Check if a particular time was requested by the pipeline.
+  // This overrides the ivar.
+  if(outInfo->Has(vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEPS()) && tsLength>0)
+    {
+    // Get the requested time step. We only supprt requests of a single time
+    // step in this reader right now
+    double *requestedTimeSteps = 
+      outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEPS());
+    
+    // find the first time value larger than requested time value
+    // this logic could be improved
+    int cnt = 0;
+    while (cnt < tsLength-1 && steps[cnt] < requestedTimeSteps[0])
+      {
+      cnt++;
+      }
+    this->ActualTimeValue = steps[cnt];
+    }
+
+  cout << "Executing with: " << this->ActualTimeValue << endl;
+
   int i, timeSet, fileSet, timeStep, timeStepInFile, fileNum;
   vtkDataArray *times;
   vtkIdList *numStepsList, *filenameNumbers;
@@ -177,10 +219,9 @@ void vtkEnSightReader::Execute()
   if ( ! this->CaseFileRead)
     {
     vtkErrorMacro("error reading case file");
-    return;
+    return 0;
     }
   
-  this->OutputsAreValid = 1;
   this->NumberOfNewOutputs = 0;
   this->NumberOfGeometryParts = 0;
   if (this->GeometryFileName)
@@ -200,7 +241,8 @@ void vtkEnSightReader::Execute()
         for (i = 1; i < times->GetNumberOfTuples(); i++)
           {
           newTime = times->GetComponent(i, 0);
-          if (newTime <= this->TimeValue && newTime > this->GeometryTimeValue)
+          if (newTime <= this->ActualTimeValue && 
+              newTime > this->GeometryTimeValue)
             {
             this->GeometryTimeValue = newTime;
             timeStep++;
@@ -247,7 +289,7 @@ void vtkEnSightReader::Execute()
               IsId(this->GeometryFileSet);
             if (collectionNum > -1)
               {
-              filenameNumbers = this->TimeSetFileNameNumbers->
+              filenameNumbers = this->FileSetFileNameNumbers->
                 GetItem(collectionNum);
               filenameNum = filenameNumbers->GetId(fileNum-1);
               this->ReplaceWildcards(fileName, filenameNum);
@@ -257,11 +299,11 @@ void vtkEnSightReader::Execute()
         }
       }
     
-    if (!this->ReadGeometryFile(fileName, timeStepInFile))
+    if (!this->ReadGeometryFile(fileName, timeStepInFile, output))
       {
       vtkErrorMacro("error reading geometry file");
       delete [] fileName;
-      return;
+      return 0;
       }
     
     delete [] fileName;
@@ -283,7 +325,8 @@ void vtkEnSightReader::Execute()
         for (i = 1; i < times->GetNumberOfTuples(); i++)
           {
           newTime = times->GetComponent(i, 0);
-          if (newTime <= this->TimeValue && newTime > this->MeasuredTimeValue)
+          if (newTime <= this->ActualTimeValue && 
+              newTime > this->MeasuredTimeValue)
             {
             this->MeasuredTimeValue = newTime;
             timeStep++;
@@ -330,7 +373,7 @@ void vtkEnSightReader::Execute()
               IsId(this->MeasuredFileSet);
             if (collectionNum > -1)
               {
-              filenameNumbers = this->TimeSetFileNameNumbers->
+              filenameNumbers = this->FileSetFileNameNumbers->
                 GetItem(fileSet);
               filenameNum = filenameNumbers->GetId(fileNum-1);
               this->ReplaceWildcards(fileName, filenameNum);
@@ -339,56 +382,75 @@ void vtkEnSightReader::Execute()
           }
         }
       }
-    if (!this->ReadMeasuredGeometryFile(fileName, timeStepInFile))
+    if (!this->ReadMeasuredGeometryFile(fileName, timeStepInFile, output))
       {
       vtkErrorMacro("error reading measured geometry file");
       delete [] fileName;
-      return;
+      return 0;
       }
     delete [] fileName;
     }
-  if (!this->CheckOutputConsistency())
-    {
-    for (i = 0; i < this->NumberOfOutputs; i++)
-      {
-      this->GetOutput(i)->Initialize();
-      }
-    return;
-    }
 
   if ((this->NumberOfVariables + this->NumberOfComplexVariables) > 0)
     {
-    if (!this->ReadVariableFiles())
+    if (!this->ReadVariableFiles(output))
       {
       vtkErrorMacro("error reading variable files");
-      return;
+      return 0;
       }
     }
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
-void vtkEnSightReader::Update()
-{
-  vtkDebugMacro("In update");;
-  int i;
-  
-  this->UpdateInformation();
-  this->UpdateData(0);
-  
-  for (i = 0; i < this->GetNumberOfOutputs(); i++)
-    {
-    if ( this->GetOutput(i) )
-      {
-      this->GetOutput(i)->DataHasBeenGenerated();
-      this->GetOutput(i)->SetUpdateExtentToWholeExtent();
-      }
-    }
-}
-//----------------------------------------------------------------------------
-void vtkEnSightReader::ExecuteInformation()
+int vtkEnSightReader::RequestInformation(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
   vtkDebugMacro("In execute information");
   this->CaseFileRead = this->ReadCaseFile();
+
+  // Convert time steps to one sorted and uniquefied list.
+  vtkstd::vector<double> timeValues;
+  if (this->GetTimeSets())
+    {
+    int numItems = this->GetTimeSets()->GetNumberOfItems();
+    for (int i=0; i<numItems; i++)
+      {
+      vtkDataArray* array = this->GetTimeSets()->GetItem(i);
+      if (array)
+        {
+        vtkIdType numTuples = array->GetNumberOfTuples();
+        for (vtkIdType j=0; j<numTuples; j++)
+          {
+          timeValues.push_back(array->GetComponent(j, 0));
+          }
+        }
+      }
+    }
+  if (timeValues.size() > 0)
+    {
+    vtkstd::sort(timeValues.begin(), timeValues.end());
+    vtkstd::vector<double> uniqueTimeValues(
+      timeValues.begin(),
+      vtkstd::unique(timeValues.begin(), timeValues.end()));
+    int numTimeValues = static_cast<int>(uniqueTimeValues.size());
+    if (numTimeValues > 0)
+      {
+      vtkInformation* outInfo = outputVector->GetInformationObject(0);
+      outInfo->Set(vtkStreamingDemandDrivenPipeline::TIME_STEPS(), 
+                   &uniqueTimeValues[0], 
+                   numTimeValues);
+      double timeRange[2];
+      timeRange[0] = uniqueTimeValues[0];
+      timeRange[1] = uniqueTimeValues[numTimeValues-1];
+      outInfo->Set(vtkStreamingDemandDrivenPipeline::TIME_RANGE(),
+                   timeRange, 2);
+      }
+    }
+  return this->CaseFileRead;
 }
 
 //----------------------------------------------------------------------------
@@ -1063,7 +1125,11 @@ int vtkEnSightReader::ReadCaseFile()
           this->ReadNextDataLine(line);
           strcpy(formatLine, " %f");
           strcpy(subLine, "");
-          sscanf(line, formatLine, &timeStep);
+          while (sscanf(line, formatLine, &timeStep) < 1)
+            {
+            // The time values may start on the next line after "time values:".
+            this->ReadNextDataLine(line);
+            }
           }
         if (firstTimeStep)
           {
@@ -1146,7 +1212,7 @@ int vtkEnSightReader::ReadCaseFile()
 }
 
 //----------------------------------------------------------------------------
-int vtkEnSightReader::ReadVariableFiles()
+int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet *output)
 {
   int i, j;
   char description[256];
@@ -1196,7 +1262,7 @@ int vtkEnSightReader::ReadVariableFiles()
       for (j = 0; j < times->GetNumberOfTuples(); j++)
         {
         newTime = times->GetComponent(j, 0);
-        if (newTime <= this->TimeValue)
+        if (newTime <= this->ActualTimeValue)
           {
           timeStep++;
           if (this->VariableTypes[i] == SCALAR_PER_MEASURED_NODE ||
@@ -1256,7 +1322,7 @@ int vtkEnSightReader::ReadVariableFiles()
           int collectionNum = this->FileSetsWithFilenameNumbers->IsId(fileSet);
           if (collectionNum > -1)
             {
-            filenameNumbers = this->TimeSetFileNameNumbers->
+            filenameNumbers = this->FileSetFileNameNumbers->
               GetItem(collectionNum);
             filenameNum = filenameNumbers->GetId(timeStep-1);
             this->ReplaceWildcards(fileName, filenameNum);
@@ -1271,35 +1337,35 @@ int vtkEnSightReader::ReadVariableFiles()
         {
         case vtkEnSightReader::SCALAR_PER_NODE:
           this->ReadScalarsPerNode(fileName, this->VariableDescriptions[i],
-                                   timeStepInFile);
+                                   timeStepInFile, output);
           break;
         case vtkEnSightReader::SCALAR_PER_MEASURED_NODE:
           this->ReadScalarsPerNode(fileName, this->VariableDescriptions[i], 
-                                   timeStepInFile, 1);
+                                   timeStepInFile, output, 1);
           break;
         case vtkEnSightReader::VECTOR_PER_NODE:
           this->ReadVectorsPerNode(fileName, this->VariableDescriptions[i],
-                                   timeStepInFile);
+                                   timeStepInFile, output);
           break;
         case vtkEnSightReader::VECTOR_PER_MEASURED_NODE:
           this->ReadVectorsPerNode(fileName, this->VariableDescriptions[i],
-                                   timeStepInFile, 1);
+                                   timeStepInFile, output, 1);
           break;
         case vtkEnSightReader::TENSOR_SYMM_PER_NODE:
           this->ReadTensorsPerNode(fileName, this->VariableDescriptions[i],
-                                   timeStepInFile);
+                                   timeStepInFile, output);
           break;
         case vtkEnSightReader::SCALAR_PER_ELEMENT:
           this->ReadScalarsPerElement(fileName, this->VariableDescriptions[i],
-                                      timeStepInFile);
+                                      timeStepInFile, output);
           break;
         case vtkEnSightReader::VECTOR_PER_ELEMENT:
           this->ReadVectorsPerElement(fileName, this->VariableDescriptions[i],
-                                      timeStepInFile);
+                                      timeStepInFile, output);
           break;
         case vtkEnSightReader::TENSOR_SYMM_PER_ELEMENT:
           this->ReadTensorsPerElement(fileName, this->VariableDescriptions[i],
-                                      timeStepInFile);
+                                      timeStepInFile, output);
           break;
         }
       }
@@ -1340,7 +1406,7 @@ int vtkEnSightReader::ReadVariableFiles()
       for (j = 0; j < times->GetNumberOfTuples(); j++)
         {
         newTime = times->GetComponent(j, 0);
-        if (newTime <= this->TimeValue)
+        if (newTime <= this->ActualTimeValue)
           {
           timeStep++;
           if (this->VariableTypes[i] == SCALAR_PER_MEASURED_NODE ||
@@ -1416,34 +1482,38 @@ int vtkEnSightReader::ReadVariableFiles()
         case vtkEnSightReader::COMPLEX_SCALAR_PER_NODE:
           this->ReadScalarsPerNode(fileName,
                                    this->ComplexVariableDescriptions[i],
-                                   timeStepInFile, 0, 2);
+                                   timeStepInFile, output, 0, 2);
           this->ReadScalarsPerNode(fileName2,
                                    this->ComplexVariableDescriptions[i],
-                                   timeStepInFile, 0, 2, 1);
+                                   timeStepInFile, output, 0, 2, 1);
           break;
         case vtkEnSightReader::COMPLEX_VECTOR_PER_NODE:
           strcpy(description, this->ComplexVariableDescriptions[i]);
           strcat(description, "_r");
-          this->ReadVectorsPerNode(fileName, description, timeStepInFile);
+          this->ReadVectorsPerNode(fileName, description, timeStepInFile,
+                                   output);
           strcpy(description, this->ComplexVariableDescriptions[i]);
           strcat(description, "_i");
-          this->ReadVectorsPerNode(fileName2, description, timeStepInFile);
+          this->ReadVectorsPerNode(fileName2, description, timeStepInFile,
+                                   output);
           break;
         case vtkEnSightReader::COMPLEX_SCALAR_PER_ELEMENT:
           this->ReadScalarsPerElement(fileName,
                                       this->ComplexVariableDescriptions[i],
-                                      timeStepInFile, 2);
+                                      timeStepInFile, output, 2);
           this->ReadScalarsPerElement(fileName2,
                                       this->ComplexVariableDescriptions[i],
-                                      timeStepInFile, 2, 1);
+                                      timeStepInFile, output, 2, 1);
           break;
         case vtkEnSightReader::COMPLEX_VECTOR_PER_ELEMENT:
           strcpy(description, this->ComplexVariableDescriptions[i]);
           strcat(description, "_r");
-          this->ReadVectorsPerElement(fileName, description, timeStepInFile);
+          this->ReadVectorsPerElement(fileName, description, timeStepInFile,
+                                      output);
           strcpy(description, this->ComplexVariableDescriptions[i]);
           strcat(description, "_i");
-          this->ReadVectorsPerElement(fileName2, description, timeStepInFile);
+          this->ReadVectorsPerElement(fileName2, description, timeStepInFile,
+                                      output);
           break;
         }
       }
@@ -1727,6 +1797,10 @@ int vtkEnSightReader::GetElementType(const char* line)
     {
     return vtkEnSightReader::QUAD8;
     }
+  else if (strncmp(line, "nfaced", 6) == 0)
+    {
+    return vtkEnSightReader::NFACED;
+    }
   else if (strncmp(line, "tetra4", 6) == 0)
     {
     return vtkEnSightReader::TETRA4;
@@ -1841,132 +1915,26 @@ void vtkEnSightReader::ReplaceWildcards(char* filename, int num)
 }
 
 //----------------------------------------------------------------------------
-// Called by constructor to set up output array.
-void vtkEnSightReader::SetNumberOfOutputsInternal(int num)
+void vtkEnSightReader::RemoveLeadingBlanks(char *line)
 {
-  int idx;
-  vtkDataObject **outputs;
-
-  // in case nothing has changed.
-  if (num == this->NumberOfOutputs)
+  int count = 0;
+  int len = static_cast<int>(strlen(line));
+  while (line[count] == ' ')
     {
-    return;
+    count++;
     }
-  
-  // Allocate new arrays.
-  outputs = new vtkDataObject *[num];
-
-  // Initialize with NULLs.
-  for (idx = 0; idx < num; ++idx)
-    {
-    outputs[idx] = NULL;
-    }
-
-  // Copy old outputs
-  for (idx = 0; idx < num && idx < this->NumberOfOutputs; ++idx)
-    {
-    outputs[idx] = this->Outputs[idx];
-    }
-  
-  // delete the previous arrays
-  if (this->Outputs)
-    {
-    delete [] this->Outputs;
-    this->Outputs = NULL;
-    this->NumberOfOutputs = 0;
-    }
-  
-  // Set the new arrays
-  this->Outputs = outputs;
-  
-  this->NumberOfOutputs = num;
-}
-
-//----------------------------------------------------------------------------
-void vtkEnSightReader::ReplaceNthOutput(int idx, vtkDataObject* newOutput)
-{
-  vtkDataObject *oldOutput;
-  
-  if (idx < 0)
-    {
-    vtkErrorMacro(<< "SetNthOutput: " << idx << ", cannot set output. ");
-    return;
-    }
-  // Expand array if necessary.
-  if (idx >= this->NumberOfOutputs)
-    {
-    this->SetNumberOfOutputsInternal(idx + 1);
-    }
-  
-  // does this change anything?
-  oldOutput = this->Outputs[idx];
-  if (newOutput == oldOutput)
-    {
-    return;
-    }
-
-  if ( !newOutput->IsA(oldOutput->GetClassName()) )
-    {
-    vtkErrorMacro("Can not replace the output with a different type.");
-    return;
-    }
-  
-  if (newOutput)
-    {
-    vtkSource *newOutputOldSource = newOutput->GetSource();
-    if (newOutputOldSource)
-      {
-      vtkErrorMacro("The new output can not have a source.");
-      return;
-      }
-    }
-
-  // disconnect first existing source-output relationship.
-  if (oldOutput)
-    {
-    oldOutput->SetSource(NULL);
-    oldOutput->UnRegister(this);
-    this->Outputs[idx] = NULL;
-    }
-  
-  if (newOutput)
-    {
-    // Register the newOutput so it does not get deleted.
-    // Don't set the link yet until previous links is disconnected.
-    newOutput->Register(this);
-    newOutput->SetSource(this);
-    }
-
-  // now actually make the link that was registered previously.
-  this->Outputs[idx] = newOutput;
-
-}
-
-int vtkEnSightReader::CheckOutputConsistency()
-{
-  if (this->NumberOfOutputs > this->NumberOfNewOutputs &&
-      ! this->InitialRead)
-    {
-    vtkErrorMacro("Cannot decrease number of outputs after initial read");
-    this->OutputsAreValid = 0;
-    }
-  
-  if (this->InitialRead)
-    {
-    this->InitialRead = 0;
-    }
-  
-  return this->OutputsAreValid;
+  memcpy(line, line+count, len-count+1);
 }
 
 //----------------------------------------------------------------------------
 vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
 {
   // Check argument range.
-  if(cellType < 0 || cellType > 15)
+  if(cellType < POINT || cellType >= NUMBER_OF_ELEMENT_TYPES)
     {
     vtkErrorMacro("Cell type " << cellType
-                  << " out of range.  Only 16 types exist.");
+                  << " out of range.  Only "
+                  << NUMBER_OF_ELEMENT_TYPES-1 << " types exist.");
     return 0;
     }
   if(index < 0 || index > this->UnstructuredPartIds->GetNumberOfIds())
@@ -1976,7 +1944,7 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
                   << " IDs exist.");
     return 0;
     }
-  
+
   // Create the container if necessary.
   if(!this->CellIds)
     {
@@ -1984,7 +1952,7 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
     }
   
   // Get the index of the actual vtkIdList requested.
-  unsigned int cellIdsIndex = index*16 + cellType;
+  unsigned int cellIdsIndex = index*NUMBER_OF_ELEMENT_TYPES + cellType;
   
   // Make sure the container is large enough for this index.
   if(cellIdsIndex+1 > this->CellIds->size())
@@ -2004,6 +1972,30 @@ vtkIdList* vtkEnSightReader::GetCellIds(int index, int cellType)
   return (*this->CellIds)[cellIdsIndex].GetPointer();
 }
 
+//----------------------------------------------------------------------------
+void vtkEnSightReader::AddToBlock(vtkMultiBlockDataSet* output, 
+                                  unsigned int blockNo,
+                                  vtkDataSet* dataset)
+{
+  vtkDataObject* blockDO = output->GetBlock(blockNo);
+  if (blockDO)
+    {
+    vtkErrorMacro("Block already has a vtkDataSet assigned to it.");
+    return;
+    }
+
+  output->SetBlock(blockNo, dataset);
+}
+
+
+//----------------------------------------------------------------------------
+vtkDataSet* vtkEnSightReader::GetDataSetFromBlock(
+  vtkMultiBlockDataSet* output,
+  unsigned int blockno)
+{
+  return vtkDataSet::SafeDownCast(output->GetBlock(blockno));
+}
+
 //----------------------------------------------------------------------------
 void vtkEnSightReader::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -2041,5 +2033,9 @@ void vtkEnSightReader::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "MinimumTimeValue: " << this->MinimumTimeValue << endl;
   os << indent << "MaximumTimeValue: " << this->MaximumTimeValue << endl;
   os << indent << "TimeSets: " << this->TimeSets << endl;
-  os << indent << "OutputsAreValid: " << this->OutputsAreValid << endl;
+  os << indent << "MeasuredFileName: " << 
+     (this->MeasuredFileName ? this->MeasuredFileName : "(none)") << endl;
+  os << indent << "MatchFileName: " << 
+     (this->MatchFileName ? this->MatchFileName : "(none)") << endl;
+  os << indent << "ParticleCoordinatesByIndex: " << this->ParticleCoordinatesByIndex << endl;
 }
