@@ -34,7 +34,7 @@
 #include "vtkUnstructuredGrid.h"
 #include "vtkExtentTranslator.h"
 
-vtkCxxRevisionMacro(vtkPDataSetReader, "$Revision: 1.37 $");
+vtkCxxRevisionMacro(vtkPDataSetReader, "$Revision: 1.39 $");
 vtkStandardNewMacro(vtkPDataSetReader);
 
 //----------------------------------------------------------------------------
@@ -932,6 +932,7 @@ int vtkPDataSetReader::RequestData(vtkInformation* request,
     output->CopyStructure(data);
     output->SetExtentTranslator(tmp);
     tmp->UnRegister(tmp);
+    output->GetFieldData()->PassData(data->GetFieldData());
     output->GetCellData()->PassData(data->GetCellData());
     output->GetPointData()->PassData(data->GetPointData());
     this->SetNumberOfPieces(0);
@@ -956,8 +957,9 @@ int vtkPDataSetReader::RequestData(vtkInformation* request,
       break;
     default:
       vtkErrorMacro("We do not handle vtkRectilinear yet.");
-      return 0;
     }
+
+  return 0;
 }
 
 
@@ -1023,6 +1025,7 @@ int vtkPDataSetReader::PolyDataExecute(vtkInformation*,
 
   append->Update();
   output->CopyStructure(append->GetOutput());
+  output->GetFieldData()->PassData(append->GetOutput()->GetFieldData());
   output->GetCellData()->PassData(append->GetOutput()->GetCellData());
   output->GetPointData()->PassData(append->GetOutput()->GetPointData());
 
@@ -1088,6 +1091,7 @@ int vtkPDataSetReader::UnstructuredGridExecute(
 
   append->Update();
   output->CopyStructure(append->GetOutput());
+  output->GetFieldData()->PassData(append->GetOutput()->GetFieldData());
   output->GetCellData()->PassData(append->GetOutput()->GetCellData());
   output->GetPointData()->PassData(append->GetOutput()->GetPointData());
 
