@@ -26,7 +26,7 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkLODProp3D, "$Revision: 1.43 $");
+vtkCxxRevisionMacro(vtkLODProp3D, "$Revision: 1.47 $");
 vtkStandardNewMacro(vtkLODProp3D);
 
 #define VTK_INDEX_NOT_IN_USE    -1
@@ -295,7 +295,7 @@ double vtkLODProp3D::GetLODIndexEstimatedRenderTime( int index )
 // are not possible
 int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p, double time )
 {
-  return this->AddLOD( m, p, (vtkProperty *) NULL, (vtkTexture *)NULL, time );
+  return this->AddLOD( m, p, NULL, NULL, time );
 }
 
 // Convenience method to set an actor LOD without a texture.
@@ -303,7 +303,7 @@ int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p, double time )
 int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p, 
                          vtkProperty *back, double time )
 {
-  return this->AddLOD( m, p, back, (vtkTexture *)NULL, time );
+  return this->AddLOD( m, p, back, NULL, time );
 }
 
 // Convenience method to set an actor LOD without a backface property.
@@ -311,22 +311,21 @@ int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p,
 int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p, 
                          vtkTexture *t, double time )
 {
-  return this->AddLOD( m, p, (vtkProperty *)NULL, t, time );
+  return this->AddLOD( m, p, NULL, t, time );
 }
 
 // Convenience method to set an actor LOD without a property.
 // Needed from tcl (for example) where null pointers are not possible
 int vtkLODProp3D::AddLOD( vtkMapper *m, vtkTexture *t, double time )
 {
-  return this->AddLOD( m, (vtkProperty *)NULL, (vtkProperty *)NULL, t, time );
+  return this->AddLOD( m, NULL, NULL, t, time );
 }
 
 // Convenience method to set an actor LOD without a texture or a property.
 // Needed from tcl (for example) where null pointers are not possible
 int vtkLODProp3D::AddLOD( vtkMapper *m, double time )
 {
-  return this->AddLOD( m, (vtkProperty *)NULL, (vtkProperty *)NULL, 
-      (vtkTexture *)NULL, time );
+  return this->AddLOD( m, NULL, NULL, NULL, time );
 } 
 
 // The real method for adding an actor LOD.
@@ -360,7 +359,7 @@ int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p,
     actor->SetTexture( t );
     }
 
-  this->LODs[index].Prop3D        = (vtkProp3D *)actor;
+  this->LODs[index].Prop3D        = actor;
   this->LODs[index].Prop3DType    = VTK_LOD_ACTOR_TYPE;
   this->LODs[index].ID            = this->CurrentIndex++;
   this->LODs[index].EstimatedTime = time;
@@ -379,7 +378,7 @@ int vtkLODProp3D::AddLOD( vtkMapper *m, vtkProperty *p,
 // Needed from tcl (for example) where null pointers are not possible
 int vtkLODProp3D::AddLOD( vtkAbstractVolumeMapper *m, double time )
 {
-  return this->AddLOD( m, (vtkVolumeProperty *)NULL, time );
+  return this->AddLOD( m, NULL, time );
 }
 
 // The real method for adding a volume LOD.
@@ -403,7 +402,7 @@ int vtkLODProp3D::AddLOD( vtkAbstractVolumeMapper *m, vtkVolumeProperty *p,
     volume->SetProperty( p );
     }
 
-  this->LODs[index].Prop3D        = (vtkProp3D *)volume;
+  this->LODs[index].Prop3D        = volume;
   this->LODs[index].Prop3DType    = VTK_LOD_VOLUME_TYPE;
   this->LODs[index].ID            = this->CurrentIndex++;
   this->LODs[index].EstimatedTime = time;
@@ -434,7 +433,7 @@ void vtkLODProp3D::SetLODMapper( int id, vtkMapper *m )
     return;
     }
 
-  ((vtkActor *)this->LODs[index].Prop3D)->SetMapper( m );
+  static_cast<vtkActor *>(this->LODs[index].Prop3D)->SetMapper( m );
 }
 
 // Get the mapper for an LOD that is an actor
@@ -456,7 +455,7 @@ void vtkLODProp3D::GetLODMapper( int id, vtkMapper **m )
     return;
     }
 
-  *m = ((vtkActor *)this->LODs[index].Prop3D)->GetMapper();
+  *m = static_cast<vtkActor *>(this->LODs[index].Prop3D)->GetMapper();
 }
 
 // Set the mapper for an LOD that is a volume
@@ -475,7 +474,7 @@ void vtkLODProp3D::SetLODMapper( int id, vtkAbstractVolumeMapper *m )
     return;
     }
 
-  ((vtkVolume *)this->LODs[index].Prop3D)->SetMapper( m );
+  static_cast<vtkVolume *>(this->LODs[index].Prop3D)->SetMapper( m );
 }
 
 // Get the mapper for an LOD that is an actor
@@ -496,7 +495,7 @@ void vtkLODProp3D::GetLODMapper( int id, vtkAbstractVolumeMapper **m )
     return;
     }
 
-  *m = ((vtkVolume *)this->LODs[index].Prop3D)->GetMapper();
+  *m = static_cast<vtkVolume *>(this->LODs[index].Prop3D)->GetMapper();
 }
 
 // Get the mapper for an LOD that is an AbstractMapper3D
@@ -513,17 +512,16 @@ vtkAbstractMapper3D *vtkLODProp3D::GetLODMapper( int id )
 
   if ( this->LODs[index].Prop3DType == VTK_LOD_ACTOR_TYPE )
     {
-        m = ((vtkActor *)this->LODs[index].Prop3D)->GetMapper();
+        m = static_cast<vtkActor *>(this->LODs[index].Prop3D)->GetMapper();
     }
   else if ( this->LODs[index].Prop3DType == VTK_LOD_VOLUME_TYPE )
     {
-        m = ((vtkVolume *)this->LODs[index].Prop3D)->GetMapper();
+        m = static_cast<vtkVolume *>(this->LODs[index].Prop3D)->GetMapper();
     }
 
   return m;
 }
 
-
 // Set the property for an LOD that is an actor
 void vtkLODProp3D::SetLODProperty( int id, vtkProperty *p )
 {
@@ -540,7 +538,7 @@ void vtkLODProp3D::SetLODProperty( int id, vtkProperty *p )
     return;
     }
 
-  ((vtkActor *)this->LODs[index].Prop3D)->SetProperty( p );
+  static_cast<vtkActor *>(this->LODs[index].Prop3D)->SetProperty( p );
 }
 
 // Get the property for an LOD that is an actor
@@ -559,7 +557,7 @@ void vtkLODProp3D::GetLODProperty( int id, vtkProperty **p )
     return;
     }
 
-  *p = ((vtkActor *)this->LODs[index].Prop3D)->GetProperty();
+  *p = static_cast<vtkActor *>(this->LODs[index].Prop3D)->GetProperty();
 }
 
 // Set the property for an LOD that is a volume
@@ -578,7 +576,7 @@ void vtkLODProp3D::SetLODProperty( int id, vtkVolumeProperty *p )
     return;
     }
 
-  ((vtkVolume *)this->LODs[index].Prop3D)->SetProperty( p );
+  static_cast<vtkVolume *>(this->LODs[index].Prop3D)->SetProperty( p );
 }
 
 // Get the property for an LOD that is an actor
@@ -597,7 +595,7 @@ void vtkLODProp3D::GetLODProperty( int id, vtkVolumeProperty **p )
     return;
     }
 
-  *p = ((vtkVolume *)this->LODs[index].Prop3D)->GetProperty();
+  *p = static_cast<vtkVolume *>(this->LODs[index].Prop3D)->GetProperty();
 }
 
 // Set the texture for an LOD that is an actor
@@ -616,7 +614,7 @@ void vtkLODProp3D::SetLODTexture( int id, vtkTexture *t )
     return;
     }
 
-  ((vtkActor *)this->LODs[index].Prop3D)->SetTexture( t );
+  static_cast<vtkActor *>(this->LODs[index].Prop3D)->SetTexture( t );
 }
 
 // Get the texture for an LOD that is an actor
@@ -635,7 +633,7 @@ void vtkLODProp3D::GetLODTexture( int id, vtkTexture **t )
     return;
     }
 
-  *t = ((vtkActor *)this->LODs[index].Prop3D)->GetTexture();
+  *t = static_cast<vtkActor *>(this->LODs[index].Prop3D)->GetTexture();
 }
 
 // Set the backface property for an LOD that is an actor
@@ -654,7 +652,7 @@ void vtkLODProp3D::SetLODBackfaceProperty( int id, vtkProperty *t )
     return;
     }
 
-  ((vtkActor *)this->LODs[index].Prop3D)->SetBackfaceProperty( t );
+  static_cast<vtkActor *>(this->LODs[index].Prop3D)->SetBackfaceProperty( t );
 }
 
 // Get the backface property for an LOD that is an actor
@@ -673,7 +671,7 @@ void vtkLODProp3D::GetLODBackfaceProperty( int id, vtkProperty **t )
     return;
     }
 
-  *t = ((vtkActor *)this->LODs[index].Prop3D)->GetBackfaceProperty();
+  *t = static_cast<vtkActor *>(this->LODs[index].Prop3D)->GetBackfaceProperty();
 }
 
 void vtkLODProp3D::EnableLOD( int id )
@@ -794,7 +792,7 @@ int vtkLODProp3D::RenderOpaqueGeometry(vtkViewport *viewport)
 }
 
 // Standard render method - render any translucent geometry in the selected LOD
-int vtkLODProp3D::RenderTranslucentGeometry(vtkViewport *viewport)
+int vtkLODProp3D::RenderTranslucentPolygonalGeometry(vtkViewport *viewport)
 {
   int retval;
 
@@ -815,7 +813,63 @@ int vtkLODProp3D::RenderTranslucentGeometry(vtkViewport *viewport)
 
   // Actually do the rendering
   retval = this->LODs[this->SelectedLODIndex].Prop3D->
-    RenderTranslucentGeometry(viewport);
+    RenderTranslucentPolygonalGeometry(viewport);
+
+  this->EstimatedRenderTime += 
+    this->LODs[this->SelectedLODIndex].Prop3D->GetEstimatedRenderTime();
+  
+  return retval;
+}
+
+// Description:
+// Does this prop have some translucent polygonal geometry?
+int vtkLODProp3D::HasTranslucentPolygonalGeometry()
+{
+  int result;
+
+  // Check if the selected index is in range
+  if ( this->SelectedLODIndex < 0 ||
+       this->SelectedLODIndex >= this->NumberOfEntries )
+    {
+    vtkErrorMacro( << "Index out of range!" );
+    return 0;
+    }
+
+  // Check if the selected index is valid
+  if ( this->LODs[this->SelectedLODIndex].ID == VTK_INDEX_NOT_IN_USE )
+    {
+    vtkErrorMacro( << "Index not valid!" );
+    return 0;
+    }
+  
+  result = this->LODs[this->SelectedLODIndex].Prop3D->HasTranslucentPolygonalGeometry();
+  
+  return result;
+}
+
+// Standard render method - render any translucent geometry in the selected LOD
+int vtkLODProp3D::RenderVolumetricGeometry(vtkViewport *viewport)
+{
+  int retval;
+
+  // Check if the selected index is in range
+  if ( this->SelectedLODIndex < 0 ||
+       this->SelectedLODIndex >= this->NumberOfEntries )
+    {
+    vtkErrorMacro( << "Index out of range!" );
+    return 0;
+    }
+
+  // Check if the selected index is valid
+  if ( this->LODs[this->SelectedLODIndex].ID == VTK_INDEX_NOT_IN_USE )
+    {
+    vtkErrorMacro( << "Index not valid!" );
+    return 0;
+    }
+
+  // Actually do the rendering
+  retval = this->LODs[this->SelectedLODIndex].Prop3D->
+    RenderVolumetricGeometry(viewport);
 
   this->EstimatedRenderTime += 
     this->LODs[this->SelectedLODIndex].Prop3D->GetEstimatedRenderTime();
@@ -823,7 +877,6 @@ int vtkLODProp3D::RenderTranslucentGeometry(vtkViewport *viewport)
   return retval;
 } 
 
-
 // Override the method from vtkProp - add to both this prop and the prop of
 // the selected LOD
 void vtkLODProp3D::AddEstimatedRenderTime( double t, vtkViewport *vp )
@@ -986,10 +1039,15 @@ void vtkLODProp3D::SetAllocatedRenderTime( double t, vtkViewport *vp )
     
     }
 
-  this->SelectedLODIndex = index;
-  this->LODs[this->SelectedLODIndex].Prop3D->SetAllocatedRenderTime( t, vp );
   this->EstimatedRenderTime = 0.0;
   this->AllocatedRenderTime = t;
+  if (index == -1)
+    {
+    return;
+    }
+
+  this->SelectedLODIndex = index;
+  this->LODs[this->SelectedLODIndex].Prop3D->SetAllocatedRenderTime( t, vp );
 
   // Push the matrix down into the selected LOD
   vtkProp3D *p = this->LODs[this->SelectedLODIndex].Prop3D;
@@ -1025,6 +1083,10 @@ void vtkLODProp3D::PrintSelf(ostream& os, vtkIndent indent)
 
 void vtkLODProp3D::GetActors(vtkPropCollection *ac)
 {
+#if 0
+  // I don't get that 1999 code, why is it limiting the actors to the one
+  // picked...
+
   vtkDebugMacro(<< "vtkLODProp3D::GetActors");
   int index;
   int lodID;
@@ -1041,6 +1103,30 @@ void vtkLODProp3D::GetActors(vtkPropCollection *ac)
     {
     ac->AddItem(this->LODs[index].Prop3D);
     }
+#else
+  int i;
+  for (i = 0; i < this->NumberOfEntries; i++)
+    {
+    if (this->LODs[i].ID != VTK_INDEX_NOT_IN_USE &&
+        vtkActor::SafeDownCast(this->LODs[i].Prop3D))
+      {
+      ac->AddItem(this->LODs[i].Prop3D);
+      }
+    }
+#endif
+}
+
+void vtkLODProp3D::GetVolumes(vtkPropCollection *ac)
+{
+  int i;
+  for (i = 0; i < this->NumberOfEntries; i++)
+    {
+    if (this->LODs[i].ID != VTK_INDEX_NOT_IN_USE &&
+        vtkVolume::SafeDownCast(this->LODs[i].Prop3D))
+      {
+      ac->AddItem(this->LODs[i].Prop3D);
+      }
+    }
 }
 
 int vtkLODProp3D::GetAutomaticPickPropIndex()
