@@ -18,11 +18,13 @@
 #include "vtkCommand.h"
 #include "vtkGraphicsFactory.h"
 #include "vtkMath.h"
+#include "vtkPainterDeviceAdapter.h"
 #include "vtkRenderWindowInteractor.h"
 #include "vtkRendererCollection.h"
+#include "vtkTimerLog.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkRenderWindow, "$Revision: 1.144 $");
+vtkCxxRevisionMacro(vtkRenderWindow, "$Revision: 1.156 $");
 
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
@@ -49,6 +51,7 @@ vtkRenderWindow::vtkRenderWindow()
   this->StereoStatus = 0;
   this->StereoCapableWindow = 0;
   this->AlphaBitPlanes = 0;
+  this->StencilCapable = 0;
   this->Interactor = NULL;
   this->AAFrames = 0;
   this->FDFrames = 0;
@@ -66,37 +69,45 @@ vtkRenderWindow::vtkRenderWindow()
   this->Renderers = vtkRendererCollection::New();
   this->NumberOfLayers = 1;
   this->CurrentCursor = VTK_CURSOR_DEFAULT;
-  this->AnaglyphColorSaturation = 0.65;
+  this->AnaglyphColorSaturation = 0.65f;
   this->AnaglyphColorMask[0] = 4;  // red
   this->AnaglyphColorMask[1] = 3;  // cyan
+  this->PainterDeviceAdapter = vtkPainterDeviceAdapter::New();
+  this->ReportGraphicErrors=0; // false
+  this->AbortCheckTime = 0.0;
 }
 
+//----------------------------------------------------------------------------
 vtkRenderWindow::~vtkRenderWindow()
 {
   this->SetInteractor(NULL);
-  
-  if (this->AccumulationBuffer) 
+
+  if (this->AccumulationBuffer)
     {
     delete [] this->AccumulationBuffer;
     this->AccumulationBuffer = NULL;
     this->AccumulationBufferSize = 0;
     }
-  if (this->ResultFrame) 
+  if (this->ResultFrame)
     {
     delete [] this->ResultFrame;
     this->ResultFrame = NULL;
     }
   this->Renderers->Delete();
+
+  this->PainterDeviceAdapter->Delete();
 }
 
-// return the correct type of RenderWindow 
+//----------------------------------------------------------------------------
+// return the correct type of RenderWindow
 vtkRenderWindow *vtkRenderWindow::New()
 {
   // First try to create the object from the vtkObjectFactory
   vtkObject* ret = vtkGraphicsFactory::CreateInstance("vtkRenderWindow");
-  return (vtkRenderWindow*)ret;
+  return static_cast<vtkRenderWindow *>(ret);
 }
 
+//----------------------------------------------------------------------------
 // Create an interactor that will work with this renderer.
 vtkRenderWindowInteractor *vtkRenderWindow::MakeRenderWindowInteractor()
 {
@@ -105,6 +116,7 @@ vtkRenderWindowInteractor *vtkRenderWindow::MakeRenderWindowInteractor()
   return this->Interactor;
 }
 
+//----------------------------------------------------------------------------
 // Set the interactor that will work with this renderer.
 void vtkRenderWindow::SetInteractor(vtkRenderWindowInteractor *rwi)
 {
@@ -112,11 +124,19 @@ void vtkRenderWindow::SetInteractor(vtkRenderWindowInteractor *rwi)
     {
     // to avoid destructor recursion
     vtkRenderWindowInteractor *temp = this->Interactor;
-    this->Interactor = rwi;    
+    this->Interactor = rwi;
     if (temp != NULL) {temp->UnRegister(this);}
-    if (this->Interactor != NULL) 
+    if (this->Interactor != NULL)
       {
       this->Interactor->Register(this);
+
+      int isize[2];
+      this->Interactor->GetSize(isize);
+      if (0 == isize[0] && 0 == isize[1])
+        {
+        this->Interactor->SetSize(this->GetSize());
+        }
+
       if (this->Interactor->GetRenderWindow() != this)
         {
         this->Interactor->SetRenderWindow(this);
@@ -125,6 +145,7 @@ void vtkRenderWindow::SetInteractor(vtkRenderWindowInteractor *rwi)
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkRenderWindow::SetSubFrames(int subFrames)
 {
   if (this->SubFrames != subFrames)
@@ -134,11 +155,13 @@ void vtkRenderWindow::SetSubFrames(int subFrames)
       {
       this->CurrentSubFrame = 0;
       }
-    vtkDebugMacro(<< this->GetClassName() << " (" << this << "): setting SubFrames to " << subFrames);
+    vtkDebugMacro(<< this->GetClassName() << " (" << this
+      << "): setting SubFrames to " << subFrames);
     this->Modified();
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkRenderWindow::SetDesiredUpdateRate(double rate)
 {
   vtkRenderer *aren;
@@ -146,11 +169,11 @@ void vtkRenderWindow::SetDesiredUpdateRate(double rate)
   if (this->DesiredUpdateRate != rate)
     {
     vtkCollectionSimpleIterator rsit;
-    for (this->Renderers->InitTraversal(rsit); 
+    for (this->Renderers->InitTraversal(rsit);
          (aren = this->Renderers->GetNextRenderer(rsit)); )
       {
       aren->SetAllocatedRenderTime(1.0/
-                                   (rate*this->Renderers->GetNumberOfItems()));
+                                  (rate*this->Renderers->GetNumberOfItems()));
       }
     this->DesiredUpdateRate = rate;
     this->Modified();
@@ -158,6 +181,7 @@ void vtkRenderWindow::SetDesiredUpdateRate(double rate)
 }
 
 
+//----------------------------------------------------------------------------
 //
 // Set the variable that indicates that we want a stereo capable window
 // be created. This method can only be called before a window is realized.
@@ -171,6 +195,7 @@ void vtkRenderWindow::SetStereoCapableWindow(int capable)
     }
 }
 
+//----------------------------------------------------------------------------
 // Turn on stereo rendering
 void vtkRenderWindow::SetStereoRender(int stereo)
 {
@@ -178,7 +203,7 @@ void vtkRenderWindow::SetStereoRender(int stereo)
     {
     return;
     }
-  
+
   if (this->StereoCapableWindow ||
       (!this->StereoCapableWindow
        && this->StereoType != VTK_STEREO_CRYSTAL_EYES))
@@ -197,9 +222,8 @@ void vtkRenderWindow::SetStereoRender(int stereo)
     }
 }
 
-
-
-// Ask each renderer owned by this RenderWindow to render its image and 
+//----------------------------------------------------------------------------
+// Ask each renderer owned by this RenderWindow to render its image and
 // synchronize this process.
 void vtkRenderWindow::Render()
 {
@@ -219,6 +243,14 @@ void vtkRenderWindow::Render()
     return;
     }
 
+  // if SetSize has not yet been called (from a script, possible off
+  // screen use, other scenarios?) then call it here with reasonable
+  // default values
+  if (0 == this->Size[0] && 0 == this->Size[1])
+    {
+    this->SetSize(300, 300);
+    }
+
   // reset the Abort flag
   this->AbortRender = 0;
   this->InRender = 1;
@@ -232,7 +264,12 @@ void vtkRenderWindow::Render()
     {
     this->Interactor->Initialize();
     }
-  
+
+  // CAUTION:
+  // This method uses this->GetSize() and allocates buffers using that size. 
+  // Remember that GetSize() will returns a size scaled by the TileScale factor.
+  // We should use GetActualSize() when we don't want the size to be scaled.
+
   // if there is a reason for an AccumulationBuffer
   if ( this->SubFrames || this->AAFrames || this->FDFrames)
     {
@@ -241,7 +278,7 @@ void vtkRenderWindow::Render()
     unsigned int bufferSize = 3*size[0]*size[1];
     // If there is not a buffer or the size is too small
     // re-allocate it
-    if( !this->AccumulationBuffer 
+    if( !this->AccumulationBuffer
         || bufferSize > this->AccumulationBufferSize)
       {
       // it is OK to delete null, no sense in two if's
@@ -252,7 +289,7 @@ void vtkRenderWindow::Render()
       memset(this->AccumulationBuffer,0,this->AccumulationBufferSize*sizeof(float));
       }
     }
-  
+
   // handle any sub frames
   if (this->SubFrames)
     {
@@ -262,7 +299,7 @@ void vtkRenderWindow::Render()
     // draw the images
     this->DoAARender();
 
-    // now accumulate the images 
+    // now accumulate the images
     if ((!this->AAFrames) && (!this->FDFrames))
       {
       p1 = this->AccumulationBuffer;
@@ -288,14 +325,14 @@ void vtkRenderWindow::Render()
         }
       delete [] p3;
       }
-    
+
     // if this is the last sub frame then convert back into unsigned char
     this->CurrentSubFrame++;
     if (this->CurrentSubFrame >= this->SubFrames)
       {
       double num;
       unsigned char *p2 = new unsigned char [3*size[0]*size[1]];
-      
+
       num = this->SubFrames;
       if (this->AAFrames)
         {
@@ -312,12 +349,18 @@ void vtkRenderWindow::Render()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
           }
         }
-      
+
       this->CurrentSubFrame = 0;
       this->CopyResultFrame();
 
@@ -338,7 +381,7 @@ void vtkRenderWindow::Render()
       double num;
       unsigned char *p2 = new unsigned char [3*size[0]*size[1]];
 
-      if (this->AAFrames) 
+      if (this->AAFrames)
         {
         num = this->AAFrames;
         }
@@ -357,34 +400,41 @@ void vtkRenderWindow::Render()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
-          *p2 = (unsigned char)(*p1/num); p1++; p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
+          *p2 = static_cast<unsigned char>(*p1/num);
+          p1++;
+          p2++;
           }
         }
-      
+
       delete [] this->AccumulationBuffer;
       this->AccumulationBuffer = NULL;
       }
-    
-    this->CopyResultFrame();
-    }  
 
-  if (this->ResultFrame) 
+    this->CopyResultFrame();
+    }
+
+  if (this->ResultFrame)
     {
     delete [] this->ResultFrame;
     this->ResultFrame = NULL;
     }
 
   this->InRender = 0;
-  this->InvokeEvent(vtkCommand::EndEvent,NULL);  
+  this->InvokeEvent(vtkCommand::EndEvent,NULL);
 }
 
+//----------------------------------------------------------------------------
 // Handle rendering any antialiased frames.
 void vtkRenderWindow::DoAARender()
 {
   int i;
-  
+
   // handle any anti aliasing
   if (this->AAFrames)
     {
@@ -410,7 +460,7 @@ void vtkRenderWindow::DoAARender()
       offsets[1] = vtkMath::Random() - 0.5;
 
       vtkCollectionSimpleIterator rsit;
-      for (this->Renderers->InitTraversal(rsit); 
+      for (this->Renderers->InitTraversal(rsit);
            (aren = this->Renderers->GetNextRenderer(rsit)); )
         {
         acam = aren->GetActiveCamera();
@@ -442,9 +492,9 @@ void vtkRenderWindow::DoAARender()
 
       // draw the images
       this->DoFDRender();
-      
+
       // restore the jitter to normal
-      for (this->Renderers->InitTraversal(rsit); 
+      for (this->Renderers->InitTraversal(rsit);
            (aren = this->Renderers->GetNextRenderer(rsit)); )
         {
         acam = aren->GetActiveCamera();
@@ -475,7 +525,7 @@ void vtkRenderWindow::DoAARender()
         }
 
 
-      // now accumulate the images 
+      // now accumulate the images
       p1 = this->AccumulationBuffer;
       if (!this->FDFrames)
         {
@@ -494,9 +544,15 @@ void vtkRenderWindow::DoAARender()
           {
           for (x = 0; x < size[0]; x++)
             {
-            *p1 += (float)*p2; p1++; p2++;
-            *p1 += (float)*p2; p1++; p2++;
-            *p1 += (float)*p2; p1++; p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
+            *p1 += static_cast<float>(*p2);
+            p1++;
+            p2++;
             }
           }
         delete [] p3;
@@ -509,12 +565,12 @@ void vtkRenderWindow::DoAARender()
     }
 }
 
-
+//----------------------------------------------------------------------------
 // Handle rendering any focal depth frames.
 void vtkRenderWindow::DoFDRender()
 {
   int i;
-  
+
   // handle any focal depth
   if (this->FDFrames)
     {
@@ -545,8 +601,8 @@ void vtkRenderWindow::DoFDRender()
       offsets[0] = vtkMath::Random(); // radius
       offsets[1] = vtkMath::Random()*360.0; // angle
 
-      // store offsets for each renderer 
-      for (this->Renderers->InitTraversal(rsit); 
+      // store offsets for each renderer
+      for (this->Renderers->InitTraversal(rsit);
            (aren = this->Renderers->GetNextRenderer(rsit)); )
         {
         acam = aren->GetActiveCamera();
@@ -574,7 +630,7 @@ void vtkRenderWindow::DoFDRender()
 
       // restore the jitter to normal
       j = 0;
-      for (this->Renderers->InitTraversal(rsit); 
+      for (this->Renderers->InitTraversal(rsit);
            (aren = this->Renderers->GetNextRenderer(rsit)); )
         {
         acam = aren->GetActiveCamera();
@@ -583,7 +639,7 @@ void vtkRenderWindow::DoFDRender()
         }
 
       // get the pixels for accumulation
-      // now accumulate the images 
+      // now accumulate the images
       p1 = this->AccumulationBuffer;
       if (this->ResultFrame)
         {
@@ -598,14 +654,20 @@ void vtkRenderWindow::DoFDRender()
         {
         for (x = 0; x < size[0]; x++)
           {
-          *p1 += (float)*p2; p1++; p2++;
-          *p1 += (float)*p2; p1++; p2++;
-          *p1 += (float)*p2; p1++; p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
+          *p1 += static_cast<float>(*p2);
+          p1++;
+          p2++;
           }
         }
       delete [] p3;
       }
-  
+
     // free memory
     delete [] orig;
     aTrans->Delete();
@@ -617,13 +679,32 @@ void vtkRenderWindow::DoFDRender()
 }
 
 
+//----------------------------------------------------------------------------
 // Handle rendering the two different views for stereo rendering.
 void vtkRenderWindow::DoStereoRender()
 {
+  vtkCollectionSimpleIterator rsit;
+
   this->Start();
   this->StereoUpdate();
+
   if (this->StereoType != VTK_STEREO_RIGHT)
     { // render the left eye
+    vtkRenderer *aren;
+    for (this->Renderers->InitTraversal(rsit);
+         (aren = this->Renderers->GetNextRenderer(rsit)); )
+      {
+      // Ugly piece of code - we need to know if the camera already
+      // exists or not. If it does not yet exist, we must reset the
+      // camera here - otherwise it will never be done (missing its
+      // oppportunity to be reset in the Render method of the
+      // vtkRenderer because it will already exist by that point...)
+      if ( !aren->IsActiveCameraCreated() )
+        {
+        aren->ResetCamera();
+        }
+      aren->GetActiveCamera()->SetLeftEye(1);
+      }
     this->Renderers->Render();
     }
 
@@ -632,12 +713,29 @@ void vtkRenderWindow::DoStereoRender()
     this->StereoMidpoint();
     if (this->StereoType != VTK_STEREO_LEFT)
       { // render the right eye
+      vtkRenderer *aren;
+      for (this->Renderers->InitTraversal(rsit);
+           (aren = this->Renderers->GetNextRenderer(rsit)); )
+        {
+        // Duplicate the ugly code here too. Of course, most
+        // times the left eye will have been rendered before
+        // the right eye, but it is possible that the user sets
+        // everything up and renders just the right eye - so we
+        // need this check here too.
+        if ( !aren->IsActiveCameraCreated() )
+          {
+          aren->ResetCamera();
+          }
+        aren->GetActiveCamera()->SetLeftEye(0);
+        }
       this->Renderers->Render();
       }
     this->StereoRenderComplete();
     }
+
 }
 
+//----------------------------------------------------------------------------
 // Add a renderer to the list of renderers.
 void vtkRenderWindow::AddRenderer(vtkRenderer *ren)
 {
@@ -652,7 +750,7 @@ void vtkRenderWindow::AddRenderer(vtkRenderer *ren)
   vtkRenderer *aren;
   vtkCollectionSimpleIterator rsit;
 
-  for (this->Renderers->InitTraversal(rsit); 
+  for (this->Renderers->InitTraversal(rsit);
        (aren = this->Renderers->GetNextRenderer(rsit)); )
     {
     aren->SetAllocatedRenderTime
@@ -660,10 +758,11 @@ void vtkRenderWindow::AddRenderer(vtkRenderer *ren)
     }
 }
 
+//----------------------------------------------------------------------------
 // Remove a renderer from the list of renderers.
 void vtkRenderWindow::RemoveRenderer(vtkRenderer *ren)
 {
-  // we are its parent 
+  // we are its parent
   this->Renderers->RemoveItem(ren);
 }
 
@@ -672,17 +771,24 @@ int vtkRenderWindow::HasRenderer(vtkRenderer *ren)
   return (ren && this->Renderers->IsItemPresent(ren));
 }
 
+//----------------------------------------------------------------------------
 int vtkRenderWindow::CheckAbortStatus()
 {
   if (!this->InAbortCheck)
     {
-    this->InAbortCheck = 1;
-    this->InvokeEvent(vtkCommand::AbortCheckEvent,NULL);
-    this->InAbortCheck = 0;
+    // Only check for abort at most one every second.
+    if (vtkTimerLog::GetUniversalTime() - this->AbortCheckTime > 1.0)
+      {
+      this->InAbortCheck = 1;
+      this->InvokeEvent(vtkCommand::AbortCheckEvent,NULL);
+      this->InAbortCheck = 0;
+      this->AbortCheckTime = vtkTimerLog::GetUniversalTime();
+      }
     }
   return this->AbortRender;
 }
 
+//----------------------------------------------------------------------------
 void vtkRenderWindow::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
@@ -693,16 +799,16 @@ void vtkRenderWindow::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Full Screen: " << (this->FullScreen ? "On\n":"Off\n");
   os << indent << "Renderers:\n";
   this->Renderers->PrintSelf(os,indent.GetNextIndent());
-  os << indent << "Stereo Capable Window Requested: " 
+  os << indent << "Stereo Capable Window Requested: "
      << (this->StereoCapableWindow ? "Yes\n":"No\n");
-  os << indent << "Stereo Render: " 
+  os << indent << "Stereo Render: "
      << (this->StereoRender ? "On\n":"Off\n");
 
-  os << indent << "Point Smoothing: " 
+  os << indent << "Point Smoothing: "
      << (this->PointSmoothing ? "On\n":"Off\n");
-  os << indent << "Line Smoothing: " 
+  os << indent << "Line Smoothing: "
      << (this->LineSmoothing ? "On\n":"Off\n");
-  os << indent << "Polygon Smoothing: " 
+  os << indent << "Polygon Smoothing: "
      << (this->PolygonSmoothing ? "On\n":"Off\n");
   os << indent << "Anti Aliased Frames: " << this->AAFrames << "\n";
   os << indent << "Abort Render: " << this->AbortRender << "\n";
@@ -719,23 +825,40 @@ void vtkRenderWindow::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "AccumulationBuffer Size " << this->AccumulationBufferSize << "\n";
   os << indent << "AlphaBitPlanes: " << (this->AlphaBitPlanes ? "On" : "Off")
      << endl;
-  
-  os << indent << "AnaglyphColorSaturation: " 
+
+  os << indent << "AnaglyphColorSaturation: "
      << this->AnaglyphColorSaturation << "\n";
-  os << indent << "AnaglyphColorMask: "  
-     << this->AnaglyphColorMask[0] << " , " 
+  os << indent << "AnaglyphColorMask: "
+     << this->AnaglyphColorMask[0] << " , "
      << this->AnaglyphColorMask[1] << "\n";
+
+  os << indent << "PainterDeviceAdapter: ";
+  if (this->PainterDeviceAdapter)
+    {
+    os << endl;
+    this->PainterDeviceAdapter->PrintSelf(os, indent.GetNextIndent());
+    }
+  else
+    {
+    os << "(none)" << endl;
+    }
+
+  os << indent << "MultiSamples: " << this->MultiSamples << "\n";
+  os << indent << "StencilCapable: " <<
+    (this->StencilCapable ? "True" : "False") << endl;
+  os << indent << "ReportGraphicErrors: "
+     << (this->ReportGraphicErrors ? "On" : "Off")<< "\n";
 }
 
-
-// Update the system, if needed, due to stereo rendering. For some stereo 
+//----------------------------------------------------------------------------
+// Update the system, if needed, due to stereo rendering. For some stereo
 // methods, subclasses might need to switch some hardware settings here.
 void vtkRenderWindow::StereoUpdate(void)
 {
   // if stereo is on and it wasn't before
   if (this->StereoRender && (!this->StereoStatus))
     {
-    switch (this->StereoType) 
+    switch (this->StereoType)
       {
       case VTK_STEREO_RED_BLUE:
         this->StereoStatus = 1;
@@ -745,14 +868,14 @@ void vtkRenderWindow::StereoUpdate(void)
         break;
       case VTK_STEREO_DRESDEN:
         this->StereoStatus = 1;
-        break;      
+        break;
       case VTK_STEREO_INTERLACED:
         this->StereoStatus = 1;
       }
     }
   else if ((!this->StereoRender) && this->StereoStatus)
     {
-    switch (this->StereoType) 
+    switch (this->StereoType)
       {
       case VTK_STEREO_RED_BLUE:
         this->StereoStatus = 0;
@@ -770,13 +893,14 @@ void vtkRenderWindow::StereoUpdate(void)
     }
 }
 
+//----------------------------------------------------------------------------
 // Intermediate method performs operations required between the rendering
 // of the left and right eye.
 void vtkRenderWindow::StereoMidpoint(void)
 {
   vtkRenderer * aren;
   /* For IceT stereo */
-  for (Renderers->InitTraversal() ; (aren = Renderers->GetNextItem()) ; ) 
+  for (Renderers->InitTraversal() ; (aren = Renderers->GetNextItem()) ; )
     {
     aren->StereoMidpoint();
     }
@@ -793,11 +917,12 @@ void vtkRenderWindow::StereoMidpoint(void)
     }
 }
 
+//----------------------------------------------------------------------------
 // Handles work required once both views have been rendered when using
 // stereo rendering.
 void vtkRenderWindow::StereoRenderComplete(void)
 {
-  switch (this->StereoType) 
+  switch (this->StereoType)
     {
     case VTK_STEREO_RED_BLUE:
       {
@@ -824,7 +949,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
         }
       p3 = result;
 
-      // now merge the two images 
+      // now merge the two images
       for (x = 0; x < size[0]; x++)
         {
         for (y = 0; y < size[1]; y++)
@@ -834,7 +959,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
           res = p2[0] + p2[1] + p2[2];
           p3[1] = 0;
           p3[2] = res/3;
-          
+
           p1 += 3;
           p2 += 3;
           p3 += 3;
@@ -878,7 +1003,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
       m0 = this->AnaglyphColorMask[0];
       m1 = this->AnaglyphColorMask[1];
 
-      for(x = 0; x < 256; x++) 
+      for(x = 0; x < 256; x++)
         {
         avecolor[x][0] = int((1.0-a)*x*0.3086);
         avecolor[x][1] = int((1.0-a)*x*0.6094);
@@ -887,34 +1012,34 @@ void vtkRenderWindow::StereoRenderComplete(void)
         satcolor[x] = int(a*x);
         }
 
-      // now merge the two images 
+      // now merge the two images
       for (x = 0; x < size[0]; x++)
         {
         for (y = 0; y < size[1]; y++)
           {
             ave0 = avecolor[p0[0]][0] + avecolor[p0[1]][1] + avecolor[p0[2]][2];
             ave1 = avecolor[p1[0]][0] + avecolor[p1[1]][1] + avecolor[p1[2]][2];
-            if (m0 & 0x4) 
+            if (m0 & 0x4)
               {
               p2[0] = satcolor[p0[0]] + ave0;
               }
-            if (m0 & 0x2) 
+            if (m0 & 0x2)
               {
               p2[1] = satcolor[p0[1]] + ave0;
               }
-            if (m0 & 0x1) 
+            if (m0 & 0x1)
               {
               p2[2] = satcolor[p0[2]] + ave0;
               }
-            if (m1 & 0x4) 
+            if (m1 & 0x4)
               {
               p2[0] = satcolor[p1[0]] + ave1;
               }
-            if (m1 & 0x2) 
+            if (m1 & 0x2)
               {
               p2[1] = satcolor[p1[1]] + ave1;
               }
-            if (m1 & 0x1) 
+            if (m1 & 0x1)
               {
               p2[2] = satcolor[p1[2]] + ave1;
               }
@@ -953,7 +1078,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
         return;
         }
 
-      // now merge the two images 
+      // now merge the two images
       p3 = result;
       for (y = 0; y < size[1]; y += 2)
         {
@@ -989,7 +1114,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
       delete [] buff;
       }
       break;
-      
+
     case VTK_STEREO_DRESDEN:
       {
       unsigned char *buff;
@@ -997,66 +1122,66 @@ void vtkRenderWindow::StereoRenderComplete(void)
       unsigned char* result;
       int *size;
       int x,y;
-      
+
       // get the size
       size = this->GetSize();
       // get the data
       buff = this->GetPixelData(0,0,size[0]-1,size[1]-1,!this->DoubleBuffer);
       p1 = this->StereoBuffer;
       p2 = buff;
-      
+
       // allocate the result
       result = new unsigned char [size[0]*size[1]*3];
-      if (!result) 
+      if (!result)
         {
         vtkErrorMacro(
           <<"Couldn't allocate memory for dresden display stereo.");
         return;
         }
-      
-      // now merge the two images 
+
+      // now merge the two images
       p3 = result;
-      
-      for (y = 0; y < size[1]; y++ ) 
+
+      for (y = 0; y < size[1]; y++ )
         {
-        for (x = 0; x < size[0]; x+=2) 
+        for (x = 0; x < size[0]; x+=2)
           {
           *p3++ = *p1++;
           *p3++ = *p1++;
           *p3++ = *p1++;
-          
+
           p3+=3;
           p1+=3;
           }
-        if( size[0] % 2 == 1 ) 
+        if( size[0] % 2 == 1 )
           {
           p3 -= 3;
           p1 -= 3;
           }
         }
-      
+
       // now the other eye
       p3 = result + 3;
       p2 += 3;
-      
-      for (y = 0; y < size[1]; y++) 
+
+      for (y = 0; y < size[1]; y++)
         {
-        for (x = 1; x < size[0]; x+=2) 
+        for (x = 1; x < size[0]; x+=2)
           {
           *p3++ = *p2++;
           *p3++ = *p2++;
           *p3++ = *p2++;
-          
+
           p3+=3;
           p2+=3;
           }
-        if( size[0] % 2 == 1 ) 
+        if( size[0] % 2 == 1 )
           {
           p3 += 3;
           p2 += 3;
           }
         }
-      
+
       this->ResultFrame = result;
       delete [] this->StereoBuffer;
       this->StereoBuffer = NULL;
@@ -1066,7 +1191,7 @@ void vtkRenderWindow::StereoRenderComplete(void)
     }
 }
 
-
+//----------------------------------------------------------------------------
 void vtkRenderWindow::CopyResultFrame(void)
 {
   if (this->ResultFrame)
@@ -1082,6 +1207,7 @@ void vtkRenderWindow::CopyResultFrame(void)
 }
 
 
+//----------------------------------------------------------------------------
 // treat renderWindow and interactor as one object.
 // it might be easier if the GetReference count method were redefined.
 void vtkRenderWindow::UnRegister(vtkObjectBase *o)
@@ -1099,11 +1225,36 @@ void vtkRenderWindow::UnRegister(vtkObjectBase *o)
       return;
       }
     }
-  
+
   this->vtkObject::UnRegister(o);
 }
 
-const char *vtkRenderWindow::GetRenderLibrary() 
+//----------------------------------------------------------------------------
+const char *vtkRenderWindow::GetRenderLibrary()
 {
   return vtkGraphicsFactory::GetRenderLibrary();
 }
+
+// Description: Return the stereo type as a character string.
+// when this method was inlined, static linking on BlueGene failed
+// (symbol referenced which is defined in discarded section)
+const char *vtkRenderWindow::GetStereoTypeAsString()
+{
+  switch ( this->StereoType )
+  {
+    case VTK_STEREO_CRYSTAL_EYES:
+      return "CrystalEyes";
+    case VTK_STEREO_RED_BLUE:
+      return "RedBlue";
+    case VTK_STEREO_LEFT:
+      return "Left";
+    case VTK_STEREO_RIGHT:
+      return "Right";
+    case VTK_STEREO_DRESDEN:
+      return "DresdenDisplay";
+    case VTK_STEREO_ANAGLYPH:
+      return "Anaglyph";
+    default:
+      return "";
+  }
+}
