@@ -46,9 +46,9 @@ extern "C" {
 /*************************************************************/
 
 /* Header file version number, required by OpenGL ABI for Linux */
-/* glxext.h last updated 2007/04/21 */
+/* glxext.h last updated 2008/08/10 */
 /* Current version at http://www.opengl.org/registry/ */
-#define GLX_GLXEXT_VERSION 19
+#define GLX_GLXEXT_VERSION 20
 
 #ifndef GLX_VERSION_1_3
 #define GLX_WINDOW_BIT                     0x00000001
@@ -346,6 +346,26 @@ extern "C" {
 #define GLX_AUX9_EXT                       0x20EB
 #endif
 
+#ifndef GLX_NV_present_video
+#define GLX_GLX_NUM_VIDEO_SLOTS_NV         0x20F0
+#endif
+
+#ifndef GLX_NV_video_out
+#define GLX_GLX_VIDEO_OUT_COLOR_NV         0x20C3
+#define GLX_GLX_VIDEO_OUT_ALPHA_NV         0x20C4
+#define GLX_GLX_VIDEO_OUT_DEPTH_NV         0x20C5
+#define GLX_GLX_VIDEO_OUT_COLOR_AND_ALPHA_NV 0x20C6
+#define GLX_GLX_VIDEO_OUT_COLOR_AND_DEPTH_NV 0x20C7
+#define GLX_GLX_VIDEO_OUT_FRAME_NV         0x20C8
+#define GLX_GLX_VIDEO_OUT_FIELD_1_NV       0x20C9
+#define GLX_GLX_VIDEO_OUT_FIELD_2_NV       0x20CA
+#define GLX_GLX_VIDEO_OUT_STACKED_FIELDS_1_2_NV 0x20CB
+#define GLX_GLX_VIDEO_OUT_STACKED_FIELDS_2_1_NV 0x20CC
+#endif
+
+#ifndef GLX_NV_swap_group
+#endif
+
 
 /*************************************************************/
 
@@ -380,7 +400,7 @@ typedef struct {
 #endif
 
 #ifndef GLEXT_64_TYPES_DEFINED
-/* This code block is duplicated in glxext.h, so must be protected */
+/* This code block is duplicated in glext.h, so must be protected */
 #define GLEXT_64_TYPES_DEFINED
 /* Define int32_t, int64_t, and uint64_t types for UST/MSC */
 /* (as used in the GLX_OML_sync_control extension). */
@@ -389,7 +409,7 @@ typedef struct {
 #elif defined(__sun__) || defined(__digital__)
 #include <inttypes.h>
 #if defined(__STDC__)
-#if defined(__arch64__)
+#if defined(__arch64__) || defined(_LP64)
 typedef long int int64_t;
 typedef unsigned long int uint64_t;
 #else
@@ -397,7 +417,7 @@ typedef long long int int64_t;
 typedef unsigned long long int uint64_t;
 #endif /* __arch64__ */
 #endif /* __STDC__ */
-#elif defined( __VMS )
+#elif defined( __VMS ) || defined(__sgi)
 #include <inttypes.h>
 #elif defined(__SCO__) || defined(__USLC__)
 #include <stdint.h>
@@ -777,6 +797,18 @@ typedef void ( * PFNGLXBINDTEXIMAGEEXTPROC) (Display *dpy, GLXDrawable drawable,
 typedef void ( * PFNGLXRELEASETEXIMAGEEXTPROC) (Display *dpy, GLXDrawable drawable, int buffer);
 #endif
 
+#ifndef GLX_NV_present_video
+#define GLX_NV_present_video 1
+#endif
+
+#ifndef GLX_NV_video_out
+#define GLX_NV_video_out 1
+#endif
+
+#ifndef GLX_NV_swap_group
+#define GLX_NV_swap_group 1
+#endif
+
 
 #ifdef __cplusplus
 }
