@@ -1,16 +1,16 @@
 /*
  * tkWinPort.h --
  *
- *  This header file handles porting issues that occur because of
- *  differences between Windows and Unix. It should be the only
- *  file that contains #ifdefs to handle different flavors of OS.
+ *      This header file handles porting issues that occur because of
+ *      differences between Windows and Unix. It should be the only
+ *      file that contains #ifdefs to handle different flavors of OS.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinPort.h,v 1.1 2004/01/07 22:32:39 barre Exp $
+ * RCS: @(#) $Id: tkWinPort.h,v 1.1 2004-01-07 22:32:38 barre Exp $
  */
 
 #ifndef _WINPORT
@@ -42,26 +42,18 @@
 #endif
 
 #include <time.h>
-#ifdef __CYGWIN__
-#    define _T(x) L##x
-#else
-#    include <tchar.h>
-#endif
+#include <tchar.h>
 
 #ifdef _MSC_VER
 #    define hypot _hypot
 #endif /* _MSC_VER */
 
-#ifndef __GNUC__
-#    define strncasecmp strnicmp
-#    define strcasecmp stricmp
-#endif
+#define strncasecmp strnicmp
+#define strcasecmp stricmp
 
 #define NBBY 8
 
-#ifndef OPEN_MAX
 #define OPEN_MAX 32
-#endif
 
 /*
  * The following define causes Tk to use its internal keysym hash table
@@ -108,7 +100,7 @@
  */
 
 #define TkpGetPixel(p) (((((p)->red >> 8) & 0xff) \
-  | ((p)->green & 0xff00) | (((p)->blue << 8) & 0xff0000)) | 0x20000000)
+        | ((p)->green & 0xff00) | (((p)->blue << 8) & 0xff0000)) | 0x20000000)
 
 /*
  * These calls implement native bitmaps which are not currently 
