@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkPort.h,v 1.1 2004/01/07 22:32:39 barre Exp $
+ * RCS: @(#) $Id: tkPort.h,v 1.1.2.2 2008-10-21 20:12:16 barre Exp $
  */
 
 #ifndef _TKPORT
@@ -26,9 +26,7 @@
 #if defined(__WIN32__) || defined(_WIN32)
 #   include "tkWinPort.h"
 #else
-#   if defined(MAC_TCL)
-#  include "tkMacPort.h"
-#   elif defined(MAC_OSX_TK)
+#   if defined(MAC_OSX_TK)
 #  include "tkMacOSXPort.h"
 #   else
 #  include "tkUnixPort.h"
