@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/VTK/VTK/Utilities/vtktiff/tif_print.c,v 1.1 2004/04/28 15:49:22 king Exp $ */
+/* $Id: tif_print.c,v 1.3 2008-03-11 19:37:29 andinet Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -60,20 +60,148 @@ static const char *orientNames[] = {
 };
 #define NORIENTNAMES    (sizeof (orientNames) / sizeof (orientNames[0]))
 
+static void
+_TIFFPrintField(FILE* fd, const TIFFFieldInfo *fip,
+                uint32 value_count, void *raw_data)
+{
+        uint32 j;
+                
+        fprintf(fd, "  %s: ", fip->field_name);
+
+        for(j = 0; j < value_count; j++) {
+                if(fip->field_type == TIFF_BYTE)
+                        fprintf(fd, "%u", ((uint8 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_UNDEFINED)
+                        fprintf(fd, "0x%x",
+                                (unsigned int) ((unsigned char *) raw_data)[j]);
+                else if(fip->field_type == TIFF_SBYTE)
+                        fprintf(fd, "%d", ((int8 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_SHORT)
+                        fprintf(fd, "%u", ((uint16 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_SSHORT)
+                        fprintf(fd, "%d", ((int16 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_LONG)
+                        fprintf(fd, "%lu",
+                                (unsigned long)((uint32 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_SLONG)
+                        fprintf(fd, "%ld", (long)((int32 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_RATIONAL
+                        || fip->field_type == TIFF_SRATIONAL
+                        || fip->field_type == TIFF_FLOAT)
+                        fprintf(fd, "%f", ((float *) raw_data)[j]);
+                else if(fip->field_type == TIFF_IFD)
+                        fprintf(fd, "0x%ulx", ((uint32 *) raw_data)[j]);
+                else if(fip->field_type == TIFF_ASCII) {
+                        fprintf(fd, "%s", (char *) raw_data);
+                        break;
+                }
+                else if(fip->field_type == TIFF_DOUBLE)
+                        fprintf(fd, "%f", ((double *) raw_data)[j]);
+                else if(fip->field_type == TIFF_FLOAT)
+                        fprintf(fd, "%f", ((float *)raw_data)[j]);
+                else {
+                        fprintf(fd, "<unsupported data type in TIFFPrint>");
+                        break;
+                }
+
+                if(j < value_count - 1)
+                        fprintf(fd, ",");
+        }
+
+        fprintf(fd, "\n");
+}
+
+static int
+_TIFFPrettyPrintField(TIFF* tif, FILE* fd, ttag_t tag,
+                      uint32 value_count, void *raw_data)
+{
+        TIFFDirectory *td = &tif->tif_dir;
+
+        switch (tag)
+        {
+                case TIFFTAG_INKSET:
+                        fprintf(fd, "  Ink Set: ");
+                        switch (*((uint16*)raw_data)) {
+                                case INKSET_CMYK:
+                                        fprintf(fd, "CMYK\n");
+                                        break;
+                                default:
+                                        fprintf(fd, "%u (0x%x)\n",
+                                                *((uint16*)raw_data),
+                                                *((uint16*)raw_data));
+                                        break;
+                        }
+                        return 1;
+                case TIFFTAG_DOTRANGE:
+                        fprintf(fd, "  Dot Range: %u-%u\n",
+                                ((uint16*)raw_data)[0], ((uint16*)raw_data)[1]);
+                        return 1;
+                case TIFFTAG_WHITEPOINT:
+                        fprintf(fd, "  White Point: %g-%g\n",
+                                ((float *)raw_data)[0], ((float *)raw_data)[1]);                        return 1;
+                case TIFFTAG_REFERENCEBLACKWHITE:
+                {
+                        uint16 i;
+
+                        fprintf(fd, "  Reference Black/White:\n");
+                        for (i = 0; i < td->td_samplesperpixel; i++)
+                        fprintf(fd, "    %2d: %5g %5g\n", i,
+                                ((float *)raw_data)[2*i+0],
+                                ((float *)raw_data)[2*i+1]);
+                        return 1;
+                }
+                case TIFFTAG_XMLPACKET:
+                {
+                        uint32 i;
+                        
+                        fprintf(fd, "  XMLPacket (XMP Metadata):\n" );
+                        for(i = 0; i < value_count; i++)
+                                fputc(((char *)raw_data)[i], fd);
+                        fprintf( fd, "\n" );
+                        return 1;
+                }
+                case TIFFTAG_RICHTIFFIPTC:
+                        /*
+                         * XXX: for some weird reason RichTIFFIPTC tag
+                         * defined as array of LONG values.
+                         */
+                        fprintf(fd,
+                                "  RichTIFFIPTC Data: <present>, %lu bytes\n",
+                                (unsigned long) value_count * 4);
+                        return 1;
+                case TIFFTAG_PHOTOSHOP:
+                        fprintf(fd, "  Photoshop Data: <present>, %lu bytes\n",
+                                (unsigned long) value_count);
+                        return 1;
+                case TIFFTAG_ICCPROFILE:
+                        fprintf(fd, "  ICC Profile: <present>, %lu bytes\n",
+                                (unsigned long) value_count);
+                        return 1;
+                case TIFFTAG_STONITS:
+                        fprintf(fd,
+                                "  Sample to Nits conversion factor: %.4e\n",
+                                *((double*)raw_data));
+                        return 1;
+        }
+
+        return 0;
+}
+
 /*
  * Print the contents of the current directory
  * to the specified stdio file stream.
  */
 void
-TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
+TEXPORT
+TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
 {
-        register TIFFDirectory *td;
+        TIFFDirectory *td = &tif->tif_dir;
         char *sep;
         uint16 i;
         long l, n;
 
-        fprintf(fd, "TIFF Directory at offset 0x%lx\n", tif->tif_diroff);
-        td = &tif->tif_dir;
+        fprintf(fd, "TIFF Directory at offset 0x%lx (%lu)\n",
+                (unsigned long)tif->tif_diroff, (unsigned long)tif->tif_diroff);
         if (TIFFFieldSet(tif,FIELD_SUBFILETYPE)) {
                 fprintf(fd, "  Subfile Type:");
                 sep = " ";
@@ -92,54 +220,18 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
         }
         if (TIFFFieldSet(tif,FIELD_IMAGEDIMENSIONS)) {
                 fprintf(fd, "  Image Width: %lu Image Length: %lu",
-                    (u_long) td->td_imagewidth, (u_long) td->td_imagelength);
+                    (unsigned long) td->td_imagewidth, (unsigned long) td->td_imagelength);
                 if (TIFFFieldSet(tif,FIELD_IMAGEDEPTH))
                         fprintf(fd, " Image Depth: %lu",
-                            (u_long) td->td_imagedepth);
+                            (unsigned long) td->td_imagedepth);
                 fprintf(fd, "\n");
         }
-
-        /* Begin Pixar */
-        if (TIFFFieldSet(tif,FIELD_IMAGEFULLWIDTH) ||
-            TIFFFieldSet(tif,FIELD_IMAGEFULLLENGTH)) {
-          fprintf(fd, "  Pixar Full Image Width: %lu Full Image Length: %lu\n",
-                  (u_long) td->td_imagefullwidth,
-                  (u_long) td->td_imagefulllength);
-        }
-        if (TIFFFieldSet(tif,FIELD_TEXTUREFORMAT))
-          _TIFFprintAsciiTag(fd, "Texture Format", td->td_textureformat);
-        if (TIFFFieldSet(tif,FIELD_WRAPMODES))
-          _TIFFprintAsciiTag(fd, "Texture Wrap Modes", td->td_wrapmodes);
-        if (TIFFFieldSet(tif,FIELD_FOVCOT))
-          fprintf(fd, "  Field of View Cotangent: %g\n", td->td_fovcot);
-        if (TIFFFieldSet(tif,FIELD_MATRIX_WORLDTOSCREEN)) {
-          typedef float Matrix[4][4];
-          Matrix*               m = (Matrix*)td->td_matrixWorldToScreen;
-          
-          fprintf(fd, "  Matrix NP:\n\t%g %g %g %g\n\t%g %g %g %g\n\t%g %g %g %g\n\t%g %g %g %g\n",
-                  (*m)[0][0], (*m)[0][1], (*m)[0][2], (*m)[0][3],
-                  (*m)[1][0], (*m)[1][1], (*m)[1][2], (*m)[1][3],
-                  (*m)[2][0], (*m)[2][1], (*m)[2][2], (*m)[2][3],
-                  (*m)[3][0], (*m)[3][1], (*m)[3][2], (*m)[3][3]);
-        }
-        if (TIFFFieldSet(tif,FIELD_MATRIX_WORLDTOCAMERA)) {
-          typedef float Matrix[4][4];
-          Matrix*               m = (Matrix*)td->td_matrixWorldToCamera;
-          
-          fprintf(fd, "  Matrix Nl:\n\t%g %g %g %g\n\t%g %g %g %g\n\t%g %g %g %g\n\t%g %g %g %g\n",
-                  (*m)[0][0], (*m)[0][1], (*m)[0][2], (*m)[0][3],
-                  (*m)[1][0], (*m)[1][1], (*m)[1][2], (*m)[1][3],
-                  (*m)[2][0], (*m)[2][1], (*m)[2][2], (*m)[2][3],
-                  (*m)[3][0], (*m)[3][1], (*m)[3][2], (*m)[3][3]);
-        }
-        /* End Pixar */
-        
         if (TIFFFieldSet(tif,FIELD_TILEDIMENSIONS)) {
                 fprintf(fd, "  Tile Width: %lu Tile Length: %lu",
-                    (u_long) td->td_tilewidth, (u_long) td->td_tilelength);
+                    (unsigned long) td->td_tilewidth, (unsigned long) td->td_tilelength);
                 if (TIFFFieldSet(tif,FIELD_TILEDEPTH))
                         fprintf(fd, " Tile Depth: %lu",
-                            (u_long) td->td_tiledepth);
+                            (unsigned long) td->td_tiledepth);
                 fprintf(fd, "\n");
         }
         if (TIFFFieldSet(tif,FIELD_RESOLUTION)) {
@@ -248,42 +340,18 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                 }
                 fprintf(fd, ">\n");
         }
-        if (TIFFFieldSet(tif,FIELD_STONITS)) {
-                fprintf(fd, "  Sample to Nits conversion factor: %.4e\n",
-                                td->td_stonits);
-        }
-#ifdef CMYK_SUPPORT
-        if (TIFFFieldSet(tif,FIELD_INKSET)) {
-                fprintf(fd, "  Ink Set: ");
-                switch (td->td_inkset) {
-                case INKSET_CMYK:
-                        fprintf(fd, "CMYK\n");
-                        break;
-                default:
-                        fprintf(fd, "%u (0x%x)\n",
-                            td->td_inkset, td->td_inkset);
-                        break;
-                }
-        }
         if (TIFFFieldSet(tif,FIELD_INKNAMES)) {
                 char* cp;
                 fprintf(fd, "  Ink Names: ");
                 i = td->td_samplesperpixel;
                 sep = "";
                 for (cp = td->td_inknames; i > 0; cp = strchr(cp,'\0')+1, i--) {
-                        fprintf(fd, "%s", sep);
+                        fputs(sep, fd);
                         _TIFFprintAscii(fd, cp);
                         sep = ", ";
                 }
+                fputs("\n", fd);
         }
-        if (TIFFFieldSet(tif,FIELD_NUMBEROFINKS))
-                fprintf(fd, " Number of Inks: %u\n", td->td_ninks);
-        if (TIFFFieldSet(tif,FIELD_DOTRANGE))
-                fprintf(fd, "  Dot Range: %u-%u\n",
-                    td->td_dotrange[0], td->td_dotrange[1]);
-        if (TIFFFieldSet(tif,FIELD_TARGETPRINTER))
-                _TIFFprintAsciiTag(fd, "Target Printer", td->td_targetprinter);
-#endif
         if (TIFFFieldSet(tif,FIELD_THRESHHOLDING)) {
                 fprintf(fd, "  Thresholding: ");
                 switch (td->td_threshholding) {
@@ -317,10 +385,20 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                         break;
                 }
         }
-#ifdef YCBCR_SUPPORT
         if (TIFFFieldSet(tif,FIELD_YCBCRSUBSAMPLING))
+        {
+            /*
+             * For hacky reasons (see tif_jpeg.c - JPEGFixupTestSubsampling),
+             * we need to fetch this rather than trust what is in our
+             * structures.
+             */
+            uint16 subsampling[2];
+
+            TIFFGetField( tif, TIFFTAG_YCBCRSUBSAMPLING, 
+                          subsampling + 0, subsampling + 1 );
                 fprintf(fd, "  YCbCr Subsampling: %u, %u\n",
-                    td->td_ycbcrsubsampling[0], td->td_ycbcrsubsampling[1]);
+                        subsampling[0], subsampling[1] );
+        }
         if (TIFFFieldSet(tif,FIELD_YCBCRPOSITIONING)) {
                 fprintf(fd, "  YCbCr Positioning: ");
                 switch (td->td_ycbcrpositioning) {
@@ -336,33 +414,9 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                         break;
                 }
         }
-        if (TIFFFieldSet(tif,FIELD_YCBCRCOEFFICIENTS))
-                fprintf(fd, "  YCbCr Coefficients: %g, %g, %g\n",
-                    td->td_ycbcrcoeffs[0],
-                    td->td_ycbcrcoeffs[1],
-                    td->td_ycbcrcoeffs[2]);
-#endif
         if (TIFFFieldSet(tif,FIELD_HALFTONEHINTS))
                 fprintf(fd, "  Halftone Hints: light %u dark %u\n",
                     td->td_halftonehints[0], td->td_halftonehints[1]);
-        if (TIFFFieldSet(tif,FIELD_ARTIST))
-                _TIFFprintAsciiTag(fd, "Artist", td->td_artist);
-        if (TIFFFieldSet(tif,FIELD_DATETIME))
-                _TIFFprintAsciiTag(fd, "Date & Time", td->td_datetime);
-        if (TIFFFieldSet(tif,FIELD_HOSTCOMPUTER))
-                _TIFFprintAsciiTag(fd, "Host Computer", td->td_hostcomputer);
-        if (TIFFFieldSet(tif,FIELD_SOFTWARE))
-                _TIFFprintAsciiTag(fd, "Software", td->td_software);
-        if (TIFFFieldSet(tif,FIELD_COPYRIGHT))
-                _TIFFprintAsciiTag(fd, "Copyright", td->td_copyright);
-        if (TIFFFieldSet(tif,FIELD_DOCUMENTNAME))
-                _TIFFprintAsciiTag(fd, "Document Name", td->td_documentname);
-        if (TIFFFieldSet(tif,FIELD_IMAGEDESCRIPTION))
-                _TIFFprintAsciiTag(fd, "Image Description", td->td_imagedescription);
-        if (TIFFFieldSet(tif,FIELD_MAKE))
-                _TIFFprintAsciiTag(fd, "Make", td->td_make);
-        if (TIFFFieldSet(tif,FIELD_MODEL))
-                _TIFFprintAsciiTag(fd, "Model", td->td_model);
         if (TIFFFieldSet(tif,FIELD_ORIENTATION)) {
                 fprintf(fd, "  Orientation: ");
                 if (td->td_orientation < NORIENTNAMES)
@@ -378,7 +432,7 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                 if (td->td_rowsperstrip == (uint32) -1)
                         fprintf(fd, "(infinite)\n");
                 else
-                        fprintf(fd, "%lu\n", (u_long) td->td_rowsperstrip);
+                        fprintf(fd, "%lu\n", (unsigned long) td->td_rowsperstrip);
         }
         if (TIFFFieldSet(tif,FIELD_MINSAMPLEVALUE))
                 fprintf(fd, "  Min Sample Value: %u\n", td->td_minsamplevalue);
@@ -405,8 +459,6 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                         break;
                 }
         }
-        if (TIFFFieldSet(tif,FIELD_PAGENAME))
-                _TIFFprintAsciiTag(fd, "Page Name", td->td_pagename);
         if (TIFFFieldSet(tif,FIELD_PAGENUMBER))
                 fprintf(fd, "  Page Number: %u-%u\n",
                     td->td_pagenumber[0], td->td_pagenumber[1]);
@@ -424,23 +476,6 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                 } else
                         fprintf(fd, "(present)\n");
         }
-#ifdef COLORIMETRY_SUPPORT
-        if (TIFFFieldSet(tif,FIELD_WHITEPOINT))
-                fprintf(fd, "  White Point: %g-%g\n",
-                    td->td_whitepoint[0], td->td_whitepoint[1]);
-        if (TIFFFieldSet(tif,FIELD_PRIMARYCHROMAS))
-                fprintf(fd, "  Primary Chromaticities: %g,%g %g,%g %g,%g\n",
-                    td->td_primarychromas[0], td->td_primarychromas[1],
-                    td->td_primarychromas[2], td->td_primarychromas[3],
-                    td->td_primarychromas[4], td->td_primarychromas[5]);
-        if (TIFFFieldSet(tif,FIELD_REFBLACKWHITE)) {
-                fprintf(fd, "  Reference Black/White:\n");
-                for (i = 0; i < td->td_samplesperpixel; i++)
-                        fprintf(fd, "    %2d: %5g %5g\n",
-                            i,
-                            td->td_refblackwhite[2*i+0],
-                            td->td_refblackwhite[2*i+1]);
-        }
         if (TIFFFieldSet(tif,FIELD_TRANSFERFUNCTION)) {
                 fprintf(fd, "  Transfer Function: ");
                 if (flags & TIFFPRINT_CURVES) {
@@ -457,40 +492,108 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                 } else
                         fprintf(fd, "(present)\n");
         }
-
-        if ( 1 )
-          {
-          int colorchannels;
-          colorchannels = td->td_samplesperpixel - td->td_extrasamples;
-          fprintf(fd, "  Color Channels: %d\n", colorchannels);
-          }
-
-#endif
-#ifdef ICC_SUPPORT
-        if (TIFFFieldSet(tif,FIELD_ICCPROFILE))
-                fprintf(fd, "  ICC Profile: <present>, %lu bytes\n",
-                    (u_long) td->td_profileLength);
-#endif
-#ifdef PHOTOSHOP_SUPPORT
-        if (TIFFFieldSet(tif,FIELD_PHOTOSHOP))
-                fprintf(fd, "  Photoshop Data: <present>, %lu bytes\n",
-                    (u_long) td->td_photoshopLength);
-#endif
-#ifdef IPTC_SUPPORT
-        if (TIFFFieldSet(tif,FIELD_RICHTIFFIPTC))
-                fprintf(fd, "  RichTIFFIPTC Data: <present>, %lu bytes\n",
-                    (u_long) td->td_richtiffiptcLength);
-#endif
-#if SUBIFD_SUPPORT
         if (TIFFFieldSet(tif, FIELD_SUBIFD)) {
                 fprintf(fd, "  SubIFD Offsets:");
                 for (i = 0; i < td->td_nsubifd; i++)
                         fprintf(fd, " %5lu", (long) td->td_subifd[i]);
                 fputc('\n', fd);
         }
-#endif
-        if (tif->tif_printdir)
-                (*tif->tif_printdir)(tif, fd, flags);
+
+        /*
+        ** Custom tag support.
+        */
+        {
+            int  i;
+            short count;
+
+            count = (short) TIFFGetTagListCount(tif);
+            for(i = 0; i < count; i++) {
+                ttag_t  tag = TIFFGetTagListEntry(tif, i);
+                const TIFFFieldInfo *fip;
+                uint16 value_count;
+                int mem_alloc = 0;
+                void *raw_data;
+
+                fip = TIFFFieldWithTag(tif, tag);
+                if(fip == NULL)
+                        continue;
+
+                if(fip->field_passcount) {
+                        if(TIFFGetField(tif, tag, &value_count, &raw_data) != 1)
+                                continue;
+                } else {
+                        if (fip->field_readcount == TIFF_VARIABLE
+                            || fip->field_readcount == TIFF_VARIABLE2)
+                                value_count = 1;
+                        else if (fip->field_readcount == TIFF_SPP)
+                                value_count = td->td_samplesperpixel;
+                        else
+                                value_count = fip->field_readcount;
+                        if ((fip->field_type == TIFF_ASCII
+                             || fip->field_readcount == TIFF_VARIABLE
+                             || fip->field_readcount == TIFF_VARIABLE2
+                             || fip->field_readcount == TIFF_SPP
+                             || value_count > 1)
+                            && fip->field_tag != TIFFTAG_PAGENUMBER
+                            && fip->field_tag != TIFFTAG_HALFTONEHINTS
+                            && fip->field_tag != TIFFTAG_YCBCRSUBSAMPLING
+                            && fip->field_tag != TIFFTAG_DOTRANGE) {
+                                if(TIFFGetField(tif, tag, &raw_data) != 1)
+                                        continue;
+                        } else if (fip->field_tag != TIFFTAG_PAGENUMBER
+                                   && fip->field_tag != TIFFTAG_HALFTONEHINTS
+                                   && fip->field_tag != TIFFTAG_YCBCRSUBSAMPLING
+                                   && fip->field_tag != TIFFTAG_DOTRANGE) {
+                                raw_data = _TIFFmalloc(
+                                        _TIFFDataSize(fip->field_type)
+                                        * value_count);
+                                mem_alloc = 1;
+                                if(TIFFGetField(tif, tag, raw_data) != 1) {
+                                        _TIFFfree(raw_data);
+                                        continue;
+                                }
+                        } else {
+                                /* 
+                                 * XXX: Should be fixed and removed, see the
+                                 * notes related to TIFFTAG_PAGENUMBER,
+                                 * TIFFTAG_HALFTONEHINTS,
+                                 * TIFFTAG_YCBCRSUBSAMPLING and
+                                 * TIFFTAG_DOTRANGE tags in tif_dir.c. */
+                                char *tmp;
+                                raw_data = _TIFFmalloc(
+                                        _TIFFDataSize(fip->field_type)
+                                        * value_count);
+                                tmp = raw_data;
+                                mem_alloc = 1;
+                                if(TIFFGetField(tif, tag, tmp,
+                                tmp + _TIFFDataSize(fip->field_type)) != 1) {
+                                        _TIFFfree(raw_data);
+                                        continue;
+                                }
+                        }
+                }
+
+                /*
+                 * Catch the tags which needs to be specially handled and
+                 * pretty print them. If tag not handled in
+                 * _TIFFPrettyPrintField() fall down and print it as any other
+                 * tag.
+                 */
+                if (_TIFFPrettyPrintField(tif, fd, tag, value_count, raw_data)) {
+                        if(mem_alloc)
+                                _TIFFfree(raw_data);
+                        continue;
+                }
+                else
+                        _TIFFPrintField(fd, fip, value_count, raw_data);
+
+                if(mem_alloc)
+                        _TIFFfree(raw_data);
+            }
+        }
+        
+        if (tif->tif_tagmethods.printdir)
+                (*tif->tif_tagmethods.printdir)(tif, fd, flags);
         if ((flags & TIFFPRINT_STRIPS) &&
             TIFFFieldSet(tif,FIELD_STRIPOFFSETS)) {
                 tstrip_t s;
@@ -500,9 +603,9 @@ TEXPORT TIFFPrintDirectory(TIFF* tif, FILE* fd, long flags)
                     isTiled(tif) ? "Tiles" : "Strips");
                 for (s = 0; s < td->td_nstrips; s++)
                         fprintf(fd, "    %3lu: [%8lu, %8lu]\n",
-                            (u_long) s,
-                            (u_long) td->td_stripoffset[s],
-                            (u_long) td->td_stripbytecount[s]);
+                            (unsigned long) s,
+                            (unsigned long) td->td_stripoffset[s],
+                            (unsigned long) td->td_stripbytecount[s]);
         }
 }
 
@@ -512,7 +615,7 @@ _TIFFprintAscii(FILE* fd, const char* cp)
         for (; *cp != '\0'; cp++) {
                 const char* tp;
 
-                if (isprint(*cp)) {
+                if (isprint((int)*cp)) {
                         fputc(*cp, fd);
                         continue;
                 }
@@ -533,3 +636,5 @@ _TIFFprintAsciiTag(FILE* fd, const char* name, const char* value)
         _TIFFprintAscii(fd, value);
         fprintf(fd, "\"\n");
 }
+
+/* vim: set ts=8 sts=8 sw=8 noet: */
