@@ -13,7 +13,9 @@
 
 =========================================================================*/
 
+#include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "vtkParse.h"
 #include "vtkConfigure.h"
@@ -22,6 +24,84 @@ int numberOfWrappedFunctions = 0;
 FunctionInfo *wrappedFunctions[1000];
 extern FunctionInfo *currentFunction;
 
+/* convert special characters in a string into their escape codes,
+   so that the string can be quoted in a source file (the specified
+   maxlen must be at least 32 chars)*/
+static const char *quote_string(const char *comment, int maxlen)
+{
+  static char *result = 0;
+  static int oldmaxlen = 0;
+  int i, j, n;
+
+  if (maxlen > oldmaxlen)
+    {
+    if (result)
+      {
+      free(result);
+      }
+    result = (char *)malloc(maxlen+1);
+    oldmaxlen = maxlen;
+    }
+
+  if (comment == NULL)
+    {
+    return "";
+    }
+
+  j = 0;
+
+  n = (int)strlen(comment);
+
+  for (i = 0; i < n; i++)
+    {
+    if (comment[i] == '\"')
+      {
+      strcpy(&result[j],"\\\"");
+      j += 2;
+      }
+    else if (comment[i] == '\\')
+      {
+      strcpy(&result[j],"\\\\");
+      j += 2;
+      }
+    else if (comment[i] == ']')
+      {
+      strcpy(&result[j],"\\\\]");
+      j += 3;
+      }
+    else if (comment[i] == '[')
+      {
+      strcpy(&result[j],"\\\\[");
+      j += 3;
+      }
+    else if (comment[i] == '\n')
+      {
+      strcpy(&result[j],"\\n");
+      j += 2;
+      }      
+    else if (isprint(comment[i]))
+      {
+      result[j] = comment[i];
+      j++;
+      }
+    else
+      {
+      sprintf(&result[j],"\\%3.3o",comment[i]);
+      j += 4;
+      }
+
+    if (j >= maxlen - 21)
+      {
+      sprintf(&result[j]," ...\\n [Truncated]\\n");
+      j += (int)strlen(" ...\\n [Truncated]\\n");
+      break;
+      }
+    }
+
+  result[j] = '\0';
+
+  return result;
+}
 
 void output_temp(FILE *fp, int i, int aType, char *Id, int count)
 {
@@ -31,7 +111,7 @@ void output_temp(FILE *fp, int i, int aType, char *Id, int count)
     fprintf(fp,"    vtkTclVoidFuncArg *temp%i = new vtkTclVoidFuncArg;\n",i);
     return;
     }
-  
+
   /* ignore void */
   if (((aType % 0x10) == 0x2)&&(!((aType % 0x1000)/0x100)))
     {
@@ -67,6 +147,7 @@ void output_temp(FILE *fp, int i, int aType, char *Id, int count)
     case 0xB:   fprintf(fp,"long long "); break;
     case 0xC:   fprintf(fp,"__int64 "); break;
     case 0xD:   fprintf(fp,"signed char "); break;
+    case 0xE:   fprintf(fp,"bool "); break;
     case 0x8: return;
     }
 
@@ -97,61 +178,60 @@ void use_hints(FILE *fp)
 {
   int  i;
 
+#define INDENT "    "
+
+  fprintf(fp,INDENT "if(temp%i)\n",MAX_ARGS);
+  fprintf(fp,INDENT "  {\n");
+  fprintf(fp,INDENT "  char tempResult[1024];\n");
+  fprintf(fp,INDENT "  *tempResult = '\\0';\n");
+
+  /* special case for double: use Tcl_PrintDouble to control precision */
+  if (currentFunction->ReturnType % 0x1000 != 0x301
+       && currentFunction->ReturnType % 0x1000 != 0x307 )
+    {
+    fprintf(fp,INDENT "  sprintf(tempResult,\"");
+    }
+
   /* use the hint */
   switch (currentFunction->ReturnType % 0x1000)
     {
     case 0x301: case 0x307:  
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
+      fprintf(fp,INDENT "  char converted[1024];\n");
+      fprintf(fp,INDENT "  *converted = '\\0';\n");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
-        fprintf(fp,"%%g ");
+        fprintf(fp,INDENT "  Tcl_PrintDouble(interp,temp%i[%i], converted);\n",MAX_ARGS,i);
+        fprintf(fp,INDENT "  strcat(tempResult, \" \");\n");
+        fprintf(fp,INDENT "  strcat(tempResult, converted);\n");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x304: case 0x305: case 0x30D:
 #ifndef VTK_USE_64BIT_IDS
     case 0x30A:
 #endif
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%i ");
         }
-      fprintf(fp,"\"");
+      break;
+
+    case 0x30E:
       for (i = 0; i < currentFunction->HintSize; i++)
         {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
+        fprintf(fp,"%%i ");
         }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x306:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%li ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
 #ifdef VTK_USE_64BIT_IDS
     case 0x30A:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
 #  if defined(_MSC_VER)
@@ -160,82 +240,42 @@ void use_hints(FILE *fp)
         fprintf(fp,"%%lli ");
 #  endif
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
 #endif
+
     case 0x30B:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%lli ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x30C:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%I64i ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x313: case 0x314: case 0x315:
 #ifndef VTK_USE_64BIT_IDS
     case 0x31A:
 #endif
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%u ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x316:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%lu ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
 #ifdef VTK_USE_64BIT_IDS
     case 0x31A:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
 #  if defined(_MSC_VER)
@@ -244,46 +284,43 @@ void use_hints(FILE *fp)
         fprintf(fp,"%%llu ");
 #  endif
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
 #endif
+
     case 0x31B:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%llu ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+
     case 0x31C:
-      fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"");
       for (i = 0; i < currentFunction->HintSize; i++)
         {
         fprintf(fp,"%%I64u ");
         }
-      fprintf(fp,"\"");
-      for (i = 0; i < currentFunction->HintSize; i++)
-        {
-        fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
-        }
-      fprintf(fp,");\n");
-      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
     }
+
+  if (currentFunction->ReturnType % 0x1000 != 0x301
+       && currentFunction->ReturnType % 0x1000 != 0x307 )
+    {
+    fprintf(fp,"\"");
+    for (i = 0; i < currentFunction->HintSize; i++)
+      {
+      fprintf(fp,",temp%i[%i]",MAX_ARGS,i);
+      }
+    fprintf(fp,");\n");
+    }
+
+  fprintf(fp,INDENT "  Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
+  fprintf(fp,INDENT "  }\n");
+  fprintf(fp,INDENT "else\n");
+  fprintf(fp,INDENT "  {\n");
+  fprintf(fp,INDENT "  Tcl_SetResult(interp, const_cast<char *>(\"\"), TCL_VOLATILE);\n");
+  fprintf(fp,INDENT "  }\n");
+
+#undef INDENT
 }
 
 void return_result(FILE *fp)
@@ -295,7 +332,11 @@ void return_result(FILE *fp)
       break;
     case 0x1: case 0x7: 
       fprintf(fp,"    char tempResult[1024];\n");
-      fprintf(fp,"    sprintf(tempResult,\"%%g\",temp%i);\n",
+       /*
+        * use tcl's print double function to support variable
+        * precision at runtime
+        */
+      fprintf(fp,"    Tcl_PrintDouble(interp,temp%i,tempResult);\n",
               MAX_ARGS); 
       fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
@@ -309,6 +350,12 @@ void return_result(FILE *fp)
               MAX_ARGS); 
       fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
+    case 0xE:
+      fprintf(fp,"    char tempResult[1024];\n");
+      fprintf(fp,"    sprintf(tempResult,\"%%i\",(int)temp%i);\n",
+              MAX_ARGS); 
+      fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
+      break;
     case 0x5:
       fprintf(fp,"    char tempResult[1024];\n");
       fprintf(fp,"    sprintf(tempResult,\"%%hi\",temp%i);\n",
@@ -399,7 +446,7 @@ void return_result(FILE *fp)
       fprintf(fp,"    Tcl_SetResult(interp, tempResult, TCL_VOLATILE);\n");
       break;
     case 0x303:
-      fprintf(fp,"    if (temp%i)\n      {\n      Tcl_SetResult(interp, (char*)temp%i, TCL_VOLATILE);\n",MAX_ARGS,MAX_ARGS); 
+      fprintf(fp,"    if (temp%i)\n      {\n      Tcl_SetResult(interp, const_cast<char *>(temp%i), TCL_VOLATILE);\n",MAX_ARGS,MAX_ARGS); 
       fprintf(fp,"      }\n    else\n      {\n");
       fprintf(fp,"      Tcl_ResetResult(interp);\n      }\n"); 
       break;
@@ -411,18 +458,18 @@ void return_result(FILE *fp)
       break;
     case 0x109:
     case 0x309:  
-      fprintf(fp,"      vtkTclGetObjectFromPointer(interp,(void *)temp%i,\"%s\");\n",MAX_ARGS,currentFunction->ReturnClass);
+      fprintf(fp,"      vtkTclGetObjectFromPointer(interp,(void *)(temp%i),\"%s\");\n",MAX_ARGS,currentFunction->ReturnClass);
       break;
 
     /* handle functions returning vectors */
     /* this is done by looking them up in a hint file */
     case 0x301: case 0x307:
-    case 0x304: case 0x305: case 0x306: case 0x30A: case 0x30B: case 0x30C: case 0x30D:
+    case 0x304: case 0x305: case 0x306: case 0x30A: case 0x30B: case 0x30C: case 0x30D: case 0x30E:
     case 0x313: case 0x314: case 0x315: case 0x316: case 0x31A: case 0x31B: case 0x31C:
       use_hints(fp);
       break;
     default:
-      fprintf(fp,"    Tcl_SetResult(interp, (char *) \"unable to return result.\", TCL_VOLATILE);\n");
+      fprintf(fp,"    Tcl_SetResult(interp, const_cast<char *>(\"unable to return result.\"), TCL_VOLATILE);\n");
       break;
     }
 }
@@ -467,35 +514,40 @@ void get_args(FILE *fp, int i)
               start_arg); 
       fprintf(fp,"    temp%i = tempi;\n",i);
       break;
+    case 0xE:
+      fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
+              start_arg); 
+      fprintf(fp,"    temp%i = tempi ? true : false;\n",i);
+      break;
     case 0x3:
       fprintf(fp,"    temp%i = *(argv[%i]);\n",i,start_arg);
       break;
     case 0x13:
       fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
               start_arg); 
-      fprintf(fp,"    temp%i = (unsigned char)tempi;\n",i);
+      fprintf(fp,"    temp%i = static_cast<unsigned char>(tempi);\n",i);
       break;
     case 0x14: case 0x1A: case 0x1B: case 0x1C:
       fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
               start_arg); 
-      fprintf(fp,"    temp%i = (unsigned int)tempi;\n",i);
+      fprintf(fp,"    temp%i = static_cast<unsigned int>(tempi);\n",i);
       break;
     case 0x15:
       fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
               start_arg); 
-      fprintf(fp,"    temp%i = (unsigned short)tempi;\n",i);
+      fprintf(fp,"    temp%i = static_cast<unsigned short>(tempi);\n",i);
       break;
     case 0x16:
       fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
               start_arg); 
-      fprintf(fp,"    temp%i = (unsigned long)tempi;\n",i);
+      fprintf(fp,"    temp%i = static_cast<unsigned long>(tempi);\n",i);
       break;
     case 0x303:
       fprintf(fp,"    temp%i = argv[%i];\n",i,start_arg);
       break;
     case 0x109:
     case 0x309:
-      fprintf(fp,"    temp%i = (%s *)(vtkTclGetPointerFromObject(argv[%i],(char *) \"%s\",interp,error));\n",i,currentFunction->ArgClasses[i],start_arg,
+      fprintf(fp,"    temp%i = (%s *)(vtkTclGetPointerFromObject(argv[%i],const_cast<char *>(\"%s\"),interp,error));\n",i,currentFunction->ArgClasses[i],start_arg,
               currentFunction->ArgClasses[i]);
       break;
     case 0x2:    
@@ -519,28 +571,33 @@ void get_args(FILE *fp, int i)
                       start_arg); 
               fprintf(fp,"    temp%i[%i] = tempi;\n",i,j);
               break;
+            case 0xE:
+              fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
+                      start_arg); 
+              fprintf(fp,"    temp%i[%i] = tempi ? true : false;\n",i,j);
+              break;
             case 0x3:
               fprintf(fp,"    temp%i[%i] = *(argv[%i]);\n",i,j,start_arg);
               break;
             case 0x13:
               fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
                       start_arg); 
-              fprintf(fp,"    temp%i[%i] = (unsigned char)tempi;\n",i,j);
+              fprintf(fp,"    temp%i[%i] = static_cast<unsigned char>(tempi);\n",i,j);
               break;
             case 0x14: case 0x1A:
               fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
                       start_arg); 
-              fprintf(fp,"    temp%i[%i] = (unsigned int)tempi;\n",i,j);
+              fprintf(fp,"    temp%i[%i] = static_cast<unsigned int>(tempi);\n",i,j);
               break;
             case 0x15:
               fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
                       start_arg); 
-              fprintf(fp,"    temp%i[%i] = (unsigned short)tempi;\n",i,j);
+              fprintf(fp,"    temp%i[%i] = static_cast<unsigned short>(tempi);\n",i,j);
               break;
             case 0x16:
               fprintf(fp,"    if (Tcl_GetInt(interp,argv[%i],&tempi) != TCL_OK) error = 1;\n",
                       start_arg); 
-              fprintf(fp,"    temp%i[%i] = (unsigned long)tempi;\n",i,j);
+              fprintf(fp,"    temp%i[%i] = static_cast<unsigned long>(tempi);\n",i,j);
               break;
             }
           start_arg++;
@@ -605,7 +662,7 @@ void outputFunction(FILE *fp, FileInfo *data)
   switch (currentFunction->ReturnType % 0x1000)
     {
     case 0x301: case 0x307:
-    case 0x304: case 0x305: case 0x306: case 0x30A: case 0x30B: case 0x30C: case 0x30D:
+    case 0x304: case 0x305: case 0x306: case 0x30A: case 0x30B: case 0x30C: case 0x30D: case 0x30E:
     case 0x313: case 0x314: case 0x315: case 0x316: case 0x31A: case 0x31B: case 0x31C:
       args_ok = currentFunction->HaveHint;
       break;
@@ -677,7 +734,7 @@ void outputFunction(FILE *fp, FileInfo *data)
         }
       else if (currentFunction->ArgTypes[i] == 0x5000)
         {
-        fprintf(fp,"vtkTclVoidFunc,(void *)temp%i",i);
+        fprintf(fp,"vtkTclVoidFunc,static_cast<void *>(temp%i)",i);
         }
       else
         {
@@ -714,7 +771,7 @@ void outputFunction(FILE *fp, FileInfo *data)
 /* print the parsed structures */
 void vtkParseOutput(FILE *fp, FileInfo *data)
 {
-  int i,j;
+  int i,j,k;
   
   fprintf(fp,"// tcl wrapper for %s object\n//\n",data->ClassName);
   fprintf(fp,"#define VTK_WRAPPING_CXX\n");
@@ -727,6 +784,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
   fprintf(fp,"#include \"%s.h\"\n\n",data->ClassName);
   fprintf(fp,"#include \"vtkTclUtil.h\"\n");
   fprintf(fp,"#include <vtkstd/stdexcept>\n");
+  fprintf(fp,"#include <vtksys/ios/sstream>\n");
   if (data->IsConcrete)
     {
     if (strcmp(data->ClassName, "vtkRenderWindowInteractor") == 0)
@@ -744,13 +802,13 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
       fprintf(fp,"#else\n");
       fprintf(fp,"  %s *temp = %s::New();\n",data->ClassName,data->ClassName);
       fprintf(fp,"#endif\n");
-      fprintf(fp,"  return ((ClientData)temp);\n}\n\n");
+      fprintf(fp,"  return static_cast<ClientData>(temp);\n}\n\n");
       }
     else
       {
       fprintf(fp,"\nClientData %sNewCommand()\n{\n",data->ClassName);
       fprintf(fp,"  %s *temp = %s::New();\n",data->ClassName,data->ClassName);
-      fprintf(fp,"  return ((ClientData)temp);\n}\n\n");
+      fprintf(fp,"  return static_cast<ClientData>(temp);\n}\n\n");
       }
     }
   
@@ -763,7 +821,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
   fprintf(fp,"  if ((argc == 2)&&(!strcmp(\"Delete\",argv[1]))&& !vtkTclInDelete(interp))\n    {\n");
   fprintf(fp,"    Tcl_DeleteCommand(interp,argv[0]);\n");
   fprintf(fp,"    return TCL_OK;\n    }\n");
-  fprintf(fp,"   return %sCppCommand((%s *)(((vtkTclCommandArgStruct *)cd)->Pointer),interp, argc, argv);\n}\n",data->ClassName,data->ClassName);
+  fprintf(fp,"   return %sCppCommand(static_cast<%s *>(static_cast<vtkTclCommandArgStruct *>(cd)->Pointer),interp, argc, argv);\n}\n",data->ClassName,data->ClassName);
   
   fprintf(fp,"\nint VTKTCL_EXPORT %sCppCommand(%s *op, Tcl_Interp *interp,\n             int argc, char *argv[])\n{\n",data->ClassName,data->ClassName);
   fprintf(fp,"  int    tempi;\n");
@@ -775,20 +833,20 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
   fprintf(fp,"  tempd = 0; tempd = tempd;\n");
   fprintf(fp,"  temps[0] = 0; temps[0] = temps[0];\n\n");
 
-  fprintf(fp,"  if (argc < 2)\n    {\n    Tcl_SetResult(interp, (char *) \"Could not find requested method.\", TCL_VOLATILE);\n    return TCL_ERROR;\n    }\n");
+  fprintf(fp,"  if (argc < 2)\n    {\n    Tcl_SetResult(interp,const_cast<char *>(\"Could not find requested method.\"), TCL_VOLATILE);\n    return TCL_ERROR;\n    }\n");
 
   /* stick in the typecasting and delete functionality here */
   fprintf(fp,"  if (!interp)\n    {\n");
   fprintf(fp,"    if (!strcmp(\"DoTypecasting\",argv[0]))\n      {\n");
   fprintf(fp,"      if (!strcmp(\"%s\",argv[1]))\n        {\n",
           data->ClassName);
-  fprintf(fp,"        argv[2] = (char *)((void *)op);\n");
+  fprintf(fp,"        argv[2] = static_cast<char *>(static_cast<void *>(op));\n");
   fprintf(fp,"        return TCL_OK;\n        }\n");
 
   /* check our superclasses */
   for (i = 0; i < data->NumberOfSuperClasses; i++)
     {
-    fprintf(fp,"      if (%sCppCommand((%s *)op,interp,argc,argv) == TCL_OK)\n        {\n",
+    fprintf(fp,"      if (%sCppCommand(static_cast<%s *>(op),interp,argc,argv) == TCL_OK)\n        {\n",
             data->SuperClasses[i],data->SuperClasses[i]);
     fprintf(fp,"        return TCL_OK;\n        }\n");      
     }
@@ -799,7 +857,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
     {
     fprintf(fp,"  if (!strcmp(\"GetSuperClassName\",argv[1]))\n");
     fprintf(fp,"    {\n");
-    fprintf(fp,"    Tcl_SetResult(interp,(char *) \"%s\", TCL_VOLATILE);\n",data->SuperClasses[0]);
+    fprintf(fp,"    Tcl_SetResult(interp,const_cast<char *>(\"%s\"), TCL_VOLATILE);\n",data->SuperClasses[0]);
     fprintf(fp,"    return TCL_OK;\n");
     fprintf(fp,"    }\n\n");      
     }
@@ -815,7 +873,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
   
   /* add the ListInstances method */
   fprintf(fp,"\n  if (!strcmp(\"ListInstances\",argv[1]))\n    {\n");
-  fprintf(fp,"    vtkTclListInstances(interp,(ClientData)%sCommand);\n",data->ClassName);
+  fprintf(fp,"    vtkTclListInstances(interp,(ClientData)(%sCommand));\n",data->ClassName);
   fprintf(fp,"    return TCL_OK;\n    }\n");
   
   /* add the ListMethods method */
@@ -868,36 +926,202 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
       }
     }
   fprintf(fp,"    return TCL_OK;\n    }\n");
+
   
+  /* add the DescribeMethods method */
+  fprintf(fp,"\n  if (!strcmp(\"DescribeMethods\",argv[1]))\n    {\n");
+  fprintf(fp,"    if(argc>3) {\n" );
+  fprintf(fp,"      Tcl_SetResult ( interp, const_cast<char*>(\"Wrong number of arguments: object DescribeMethods <MethodName>\"), TCL_VOLATILE ); \n" );
+  fprintf(fp,"      return TCL_ERROR;\n }\n" );
+  
+  fprintf(fp,"    if(argc==2) {\n" );
+  // Return a list of methods
+  fprintf(fp,"\n  Tcl_DString dString, dStringParent;\n");
+  fprintf(fp,"\n  Tcl_DStringInit ( &dString );\n" );
+  fprintf(fp,"\n  Tcl_DStringInit ( &dStringParent );\n" );
+  /* recurse up the tree */
+  for (i = 0; i < data->NumberOfSuperClasses; i++)
+    {
+    fprintf(fp,"    %sCppCommand(op,interp,argc,argv);\n",
+            data->SuperClasses[i]);
+    /* append the result to our string */
+    fprintf(fp,"    Tcl_DStringGetResult ( interp, &dStringParent );\n" );
+    fprintf(fp,"    Tcl_DStringAppend ( &dString, Tcl_DStringValue ( &dStringParent ), -1 );\n" );
+    }
+  for (k = 0; k < numberOfWrappedFunctions; k++)
+    {
+      currentFunction = wrappedFunctions[k];
+      if(currentFunction->IsLegacy)
+        {
+          fprintf(fp,"#if !defined(VTK_LEGACY_REMOVE)\n");
+        }
+      fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", currentFunction->Name );
+     if(currentFunction->IsLegacy)
+        {
+          fprintf(fp,"#endif\n");
+        }
+    }
+  fprintf(fp,"  Tcl_DStringResult ( interp, &dString );\n" );
+  fprintf(fp,"  Tcl_DStringFree ( &dString );\n" );
+  fprintf(fp,"  Tcl_DStringFree ( &dStringParent );\n" );
+  fprintf(fp,"    return TCL_OK;\n    }\n");
+
+  /* Now handle if we are asked for a specific function */
+  fprintf(fp,"    if(argc==3) {\n" );
+  fprintf(fp,"      Tcl_DString dString;\n");
+  if (data->NumberOfSuperClasses > 0)
+  {
+    fprintf(fp,"      int SuperClassStatus;\n" );
+  }
+  /* recurse up the tree */
+  for (i = 0; i < data->NumberOfSuperClasses; i++)
+    {
+    fprintf(fp,"    SuperClassStatus = %sCppCommand(op,interp,argc,argv);\n",
+            data->SuperClasses[i]);
+    fprintf(fp,"    if ( SuperClassStatus == TCL_OK ) { return TCL_OK; }\n" );
+    }
+  // Now we handle it ourselves
+  for (k = 0; k < numberOfWrappedFunctions; k++)
+    {
+      currentFunction = wrappedFunctions[k];
+      if(currentFunction->IsLegacy)
+        {
+          fprintf(fp,"#if !defined(VTK_LEGACY_REMOVE)\n");
+        }
+      fprintf(fp,"    /* Starting function: %s */\n", currentFunction->Name );
+      fprintf(fp,"    if ( strcmp ( argv[2], \"%s\" ) == 0 ) {\n", currentFunction->Name );
+      fprintf(fp,"    Tcl_DStringInit ( &dString );\n" );
+      
+      fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", currentFunction->Name );
+      
+      /* calc the total required args */
+      fprintf(fp,"    /* Arguments */\n" );
+      fprintf(fp,"    Tcl_DStringStartSublist ( &dString );\n" );
+      for (i = 0; i < currentFunction->NumberOfArguments; i++)
+        {
+          int argtype;
+          if (currentFunction->ArgTypes[i] == 0x5000)
+            {
+              fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"function\" );\n" );
+              continue;
+            }
+          
+          argtype = currentFunction->ArgTypes[i] % 0x1000;
+          switch (argtype)
+            {
+            case 0x301:
+            case 0x307:
+              /* Vector */
+              fprintf(fp,"    Tcl_DStringStartSublist ( &dString );\n" );
+              for (j = 0; j < currentFunction->ArgCounts[i]; j++) 
+                {
+                  fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"float\" );\n" );
+                }
+              fprintf(fp,"    Tcl_DStringEndSublist ( &dString );\n" );
+              break;
+            case 0x304:
+              /* Vector */
+              fprintf(fp,"    Tcl_DStringStartSublist ( &dString );\n" );
+              for (j = 0; j < currentFunction->ArgCounts[i]; j++) 
+                {
+                  fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"int\" );\n" );
+                }
+              fprintf(fp,"    Tcl_DStringEndSublist ( &dString );\n" );
+              break;
+            case 0x30A:
+              /* Vector */
+              fprintf(fp,"    Tcl_DStringStartSublist ( &dString );\n" );
+              for (j = 0; j < currentFunction->ArgCounts[i]; j++) 
+                {
+                  fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"int\" );\n" );
+                }
+              fprintf(fp,"    Tcl_DStringEndSublist ( &dString );\n" );
+              break;
+            case 0x30B: case 0x30C:
+              /* Vector */
+              fprintf(fp,"    Tcl_DStringStartSublist ( &dString );\n" );
+              for (j = 0; j < currentFunction->ArgCounts[i]; j++) 
+                {
+                  fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"int\" );\n" );
+                }
+              fprintf(fp,"    Tcl_DStringEndSublist ( &dString );\n" );
+              break;
+            case 0x109:
+            case 0x309: fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", currentFunction->ArgClasses[i] ); break;
+            case 0x302:
+            case 0x303: fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"string\" );\n" ); break;
+            case 0x1:
+            case 0x7:   fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"float\" );\n" ); break;
+            case 0xD:
+            case 0xA:
+            case 0x1B:
+            case 0xB:
+            case 0x1C:
+            case 0xC:
+            case 0x14:
+            case 0x4:
+            case 0x15:
+            case 0x5:
+            case 0x16:
+            case 0x6:   fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"int\" );\n" ); break;
+            case 0x3:   fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"char\" );\n" ); break;
+            case 0x13:  fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"int\" );\n" ); break;
+            case 0xE:   fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"bool\" );\n" ); break;
+            }
+        }
+      fprintf(fp,"    Tcl_DStringEndSublist ( &dString );\n" );
+
+     /* Documentation */
+     fprintf(fp,"    /* Documentation for %s */\n", currentFunction->Name );
+     fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", quote_string ( currentFunction->Comment, 500 ) );
+     fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", quote_string ( currentFunction->Signature, 500 ) );
+     fprintf(fp,"    Tcl_DStringAppendElement ( &dString, \"%s\" );\n", quote_string ( data->ClassName, 500 ) );
+     fprintf(fp,"    /* Closing for %s */\n\n", currentFunction->Name );
+     fprintf(fp,"    Tcl_DStringResult ( interp, &dString );\n" );
+     fprintf(fp,"    Tcl_DStringFree ( &dString );\n" );
+     fprintf(fp,"    return TCL_OK;\n    }\n");
+
+     if(currentFunction->IsLegacy)
+        {
+        fprintf(fp,"#endif\n");
+        }
+    }
+  /* Didn't find anything, return an error */
+  fprintf(fp,"   Tcl_SetResult ( interp, const_cast<char*>(\"Could not find method\"), TCL_VOLATILE ); \n" );
+  fprintf(fp,"   return TCL_ERROR;\n" );
+  fprintf(fp,"   }\n" );
+  fprintf(fp," }\n" );
+
+
   /* try superclasses */
   for (i = 0; i < data->NumberOfSuperClasses; i++)
     {
-    fprintf(fp,"\n  if (%sCppCommand((%s *)op,interp,argc,argv) == TCL_OK)\n",
+    fprintf(fp,"\n  if (%sCppCommand(static_cast<%s *>(op),interp,argc,argv) == TCL_OK)\n",
             data->SuperClasses[i], data->SuperClasses[i]);
     fprintf(fp,"    {\n    return TCL_OK;\n    }\n");
     }
-  
+
+
   /* Add the Print method to vtkObjectBase. */
   if (!strcmp("vtkObjectBase",data->ClassName))
     {
     fprintf(fp,"  if ((!strcmp(\"Print\",argv[1]))&&(argc == 2))\n    {\n");
-    fprintf(fp,"    ostrstream buf_with_warning_C4701;\n");
+    fprintf(fp,"    vtksys_ios::ostringstream buf_with_warning_C4701;\n");
     fprintf(fp,"    op->Print(buf_with_warning_C4701);\n");
     fprintf(fp,"    buf_with_warning_C4701.put('\\0');\n");
-    fprintf(fp,"    Tcl_SetResult(interp,buf_with_warning_C4701.str(),\n");
+    fprintf(fp,"    Tcl_SetResult(interp,const_cast<char *>(buf_with_warning_C4701.str().c_str()),\n");
     fprintf(fp,"      TCL_VOLATILE);\n");
-    fprintf(fp,"    buf_with_warning_C4701.rdbuf()->freeze(0);\n");
     fprintf(fp,"    return TCL_OK;\n    }\n");
     /* Add the PrintRevisions method to vtkObjectBase. */
     fprintf(fp,"  if ((!strcmp(\"PrintRevisions\",argv[1]))&&(argc == 2))\n    {\n");
-    fprintf(fp,"    ostrstream buf_with_warning_C4701;\n");
+    fprintf(fp,"    vtksys_ios::ostringstream buf_with_warning_C4701;\n");
     fprintf(fp,"    op->PrintRevisions(buf_with_warning_C4701);\n");
     fprintf(fp,"    buf_with_warning_C4701.put('\\0');\n");
-    fprintf(fp,"    Tcl_SetResult(interp,buf_with_warning_C4701.str(),\n");
+    fprintf(fp,"    Tcl_SetResult(interp,const_cast<char *>(buf_with_warning_C4701.str().c_str()),\n");
     fprintf(fp,"      TCL_VOLATILE);\n");
-    fprintf(fp,"    delete [] buf_with_warning_C4701.str();\n");
     fprintf(fp,"    return TCL_OK;\n    }\n");    
     }
+
   /* Add the AddObserver method to vtkObject. */
   if (!strcmp("vtkObject",data->ClassName))
     {
@@ -921,8 +1145,15 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
     fprintf(fp,"      return TCL_OK;\n      }\n");
     fprintf(fp,"    }\n");
     }
-  fprintf(fp,"\n  if ((argc >= 2)&&(!strstr(interp->result,\"Object named:\")))\n    {\n");
-  fprintf(fp,"    char temps2[256];\n    sprintf(temps2,\"Object named: %%s, could not find requested method: %%s\\nor the method was called with incorrect arguments.\\n\",argv[0],argv[1]);\n    Tcl_AppendResult(interp,temps2,NULL);\n    }\n");
+
+  // i.e. If this is vtkObjectBase (or whatever the top of the class hierarchy will be)
+  // then report the error
+  if (data->NumberOfSuperClasses == 0)
+    {
+    fprintf(fp,"\n  if (argc >= 2)\n    {\n");
+    fprintf(fp,"    char temps2[256];\n    sprintf(temps2,\"Object named: %%s, could not find requested method: %%s\\nor the method was called with incorrect arguments.\\n\",argv[0],argv[1]);\n    Tcl_SetResult(interp,temps2,TCL_VOLATILE);\n    return TCL_ERROR;\n    }\n");
+    }
+
   fprintf(fp,"    }\n");
   fprintf(fp,"  catch (vtkstd::exception &e)\n");
   fprintf(fp,"    {\n");
@@ -930,5 +1161,4 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
   fprintf(fp,"    return TCL_ERROR;\n");
   fprintf(fp,"    }\n");
   fprintf(fp,"  return TCL_ERROR;\n}\n");
-
 }
